# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
SQLAlchemy models for vsm data.
"""

from sqlalchemy import Column, Integer
from sqlalchemy import BigInteger, String
from sqlalchemy import Text
from sqlalchemy import Float
from sqlalchemy.exc import IntegrityError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import ForeignKey, DateTime, Boolean
from sqlalchemy.orm import relationship, backref, object_mapper

from vsm.db.sqlalchemy.session import get_session
from vsm import exception
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.openstack.common import timeutils

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS
BASE = declarative_base()


class VsmBase(object):
    """Base class for Vsm Models."""
    __table_args__ = {'mysql_engine': 'InnoDB'}
    __table_initialized__ = False
    created_at = Column(DateTime, default=timeutils.utcnow)
    updated_at = Column(DateTime, onupdate=timeutils.utcnow)
    deleted_at = Column(DateTime)
    deleted = Column(Boolean, default=False)
    metadata = None

    def __init__(self):
        self._i = None

    def save(self, session=None):
        """Save this object."""
        if not session:
            session = get_session()
        session.add(self)
        try:
            session.flush()
        except IntegrityError, e:
            if str(e).endswith('is not unique'):
                raise exception.Duplicate(str(e))
            else:
                raise

    def delete(self, session=None):
        """Delete this object."""
        self.deleted = True
        self.deleted_at = timeutils.utcnow()
        self.save(session=session)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __getitem__(self, key):
        return getattr(self, key)

    def get(self, key, default=None):
        return getattr(self, key, default)

    def __iter__(self):
        self._i = iter(object_mapper(self).columns)
        return self

    def next(self):
        n = self._i.next().name
        return n, getattr(self, n)

    def update(self, values):
        """Make the model object behave like a dict."""
        for k, v in values.iteritems():
            setattr(self, k, v)

    def iteritems(self):
        """Make the model object behave like a dict.

        Includes attributes from joins."""
        local = dict(self)
        joined = dict([(k, v) for k, v in self.__dict__.iteritems()
                      if not k[0] == '_'])
        local.update(joined)
        return local.iteritems()

class Service(BASE, VsmBase):
    """Represents a running service on a host."""

    __tablename__ = 'services'
    id = Column(Integer, primary_key=True)
    host = Column(String(255))  # , ForeignKey('hosts.id'))
    binary = Column(String(255))
    topic = Column(String(255))
    report_count = Column(Integer, nullable=False, default=0)
    disabled = Column(Boolean, default=False)
    availability_zone = Column(String(255), default='vsm')

class VsmNode(BASE, VsmBase):
    """Represents a running vsm service on a host."""

    __tablename__ = 'vsm_nodes'
    id = Column(Integer, primary_key=True)
    service_id = Column(Integer, ForeignKey('services.id'), nullable=True)

class Hardware(BASE, VsmBase):
    """Represents a block storage device that can be attached to a vm."""
    __tablename__ = 'storages'
    id = Column(String(36), primary_key=True)

    @property
    def name(self):
        return FLAGS.storage_name_template % self.id

    ec2_id = Column(Integer)
    user_id = Column(String(255))
    project_id = Column(String(255))

    snapshot_id = Column(String(36))

    host = Column(String(255))  # , ForeignKey('hosts.id'))
    size = Column(Integer, default=0)
    availability_zone = Column(String(255))  # TODO(vish): foreign key?
    instance_uuid = Column(String(36))
    mountpoint = Column(String(255))
    attach_time = Column(String(255))  # TODO(vish): datetime
    status = Column(String(255))  # TODO(vish): enum?
    attach_status = Column(String(255))  # TODO(vish): enum

    scheduled_at = Column(DateTime)
    launched_at = Column(DateTime)
    terminated_at = Column(DateTime)

    display_name = Column(String(255))
    display_description = Column(String(255))

    provider_location = Column(String(255))
    provider_auth = Column(String(255))

    storage_type_id = Column(String(36))
    source_volid = Column(String(36))

class HardwareMetadata(BASE, VsmBase):
    """Represents a metadata key/value pair for a storage."""
    __tablename__ = 'storage_metadata'
    id = Column(Integer, primary_key=True)
    key = Column(String(255))
    value = Column(String(255))
    storage_id = Column(String(36), ForeignKey('storages.id'), nullable=False)
    storage = relationship(Hardware, backref="storage_metadata",
                          foreign_keys=storage_id,
                          primaryjoin='and_('
                          'HardwareMetadata.storage_id == Hardware.id,'
                          'HardwareMetadata.deleted == False)')

class HardwareTypes(BASE, VsmBase):
    """Represent possible storage_types of storages offered."""
    __tablename__ = "storage_types"
    id = Column(String(36), primary_key=True)
    name = Column(String(255))

    storages = relationship(Hardware,
                           backref=backref('storage_type', uselist=False),
                           foreign_keys=id,
                           primaryjoin='and_('
                           'Hardware.storage_type_id == HardwareTypes.id, '
                           'HardwareTypes.deleted == False)')

class HardwareTypeExtraSpecs(BASE, VsmBase):
    """Represents additional specs as key/value pairs for a storage_type."""
    __tablename__ = 'storage_type_extra_specs'
    id = Column(Integer, primary_key=True)
    key = Column(String(255))
    value = Column(String(255))
    storage_type_id = Column(String(36),
                            ForeignKey('storage_types.id'),
                            nullable=False)
    storage_type = relationship(
        HardwareTypes,
        backref="extra_specs",
        foreign_keys=storage_type_id,
        primaryjoin='and_('
        'HardwareTypeExtraSpecs.storage_type_id == HardwareTypes.id,'
        'HardwareTypeExtraSpecs.deleted == False)'
    )

class HardwareGlanceMetadata(BASE, VsmBase):
    """Glance metadata for a bootable storage."""
    __tablename__ = 'storage_glance_metadata'
    id = Column(Integer, primary_key=True, nullable=False)
    storage_id = Column(String(36), ForeignKey('storages.id'))
    snapshot_id = Column(String(36), ForeignKey('snapshots.id'))
    key = Column(String(255))
    value = Column(Text)
    storage = relationship(Hardware, backref="storage_glance_metadata",
                          foreign_keys=storage_id,
                          primaryjoin='and_('
                          'HardwareGlanceMetadata.storage_id == Hardware.id,'
                          'HardwareGlanceMetadata.deleted == False)')

class Quota(BASE, VsmBase):
    """Represents a single quota override for a project.

    If there is no row for a given project id and resource, then the
    default for the quota class is used.  If there is no row for a
    given quota class and resource, then the default for the
    deployment is used. If the row is present but the hard limit is
    Null, then the resource is unlimited.
    """

    __tablename__ = 'quotas'
    id = Column(Integer, primary_key=True)

    project_id = Column(String(255), index=True)

    resource = Column(String(255))
    hard_limit = Column(Integer, nullable=True)

class QuotaClass(BASE, VsmBase):
    """Represents a single quota override for a quota class.

    If there is no row for a given quota class and resource, then the
    default for the deployment is used.  If the row is present but the
    hard limit is Null, then the resource is unlimited.
    """

    __tablename__ = 'quota_classes'
    id = Column(Integer, primary_key=True)

    class_name = Column(String(255), index=True)

    resource = Column(String(255))
    hard_limit = Column(Integer, nullable=True)

class QuotaUsage(BASE, VsmBase):
    """Represents the current usage for a given resource."""

    __tablename__ = 'quota_usages'
    id = Column(Integer, primary_key=True)

    project_id = Column(String(255), index=True)
    resource = Column(String(255))

    in_use = Column(Integer)
    reserved = Column(Integer)

    @property
    def total(self):
        return self.in_use + self.reserved

    until_refresh = Column(Integer, nullable=True)

class Reservation(BASE, VsmBase):
    """Represents a resource reservation for quotas."""

    __tablename__ = 'reservations'
    id = Column(Integer, primary_key=True)
    uuid = Column(String(36), nullable=False)

    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=False)

    project_id = Column(String(255), index=True)
    resource = Column(String(255))

    delta = Column(Integer)
    expire = Column(DateTime, nullable=False)

class Migration(BASE, VsmBase):
    """Represents a running host-to-host migration."""
    __tablename__ = 'migrations'
    id = Column(Integer, primary_key=True, nullable=False)
    # NOTE(tr3buchet): the ____compute variables are instance['host']
    source_compute = Column(String(255))
    dest_compute = Column(String(255))
    # NOTE(tr3buchet): dest_host, btw, is an ip address
    dest_host = Column(String(255))
    old_instance_type_id = Column(Integer())
    new_instance_type_id = Column(Integer())
    instance_uuid = Column(String(255),
                           ForeignKey('instances.uuid'),
                           nullable=True)
    #TODO(_cerberus_): enum
    status = Column(String(255))

class SMFlavors(BASE, VsmBase):
    """Represents a flavor for SM storages."""
    __tablename__ = 'sm_flavors'
    id = Column(Integer(), primary_key=True)
    label = Column(String(255))
    description = Column(String(255))

class SMBackendConf(BASE, VsmBase):
    """Represents the connection to the backend for SM."""
    __tablename__ = 'sm_backend_config'
    id = Column(Integer(), primary_key=True)
    flavor_id = Column(Integer, ForeignKey('sm_flavors.id'), nullable=False)
    sr_uuid = Column(String(255))
    sr_type = Column(String(255))
    config_params = Column(String(2047))

class SMHardware(BASE, VsmBase):
    __tablename__ = 'sm_storage'
    id = Column(String(36), ForeignKey(Hardware.id), primary_key=True)
    backend_id = Column(Integer, ForeignKey('sm_backend_config.id'),
                        nullable=False)
    vdi_uuid = Column(String(255))

class Backup(BASE, VsmBase):
    """Represents a backup of a storage to Swift."""
    __tablename__ = 'backups'
    id = Column(String(36), primary_key=True)

    @property
    def name(self):
        return FLAGS.backup_name_template % self.id

    user_id = Column(String(255), nullable=False)
    project_id = Column(String(255), nullable=False)

    storage_id = Column(String(36), nullable=False)
    host = Column(String(255))
    availability_zone = Column(String(255))
    display_name = Column(String(255))
    display_description = Column(String(255))
    container = Column(String(255))
    status = Column(String(255))
    fail_reason = Column(String(255))
    service_metadata = Column(String(255))
    service = Column(String(255))
    size = Column(Integer, default=0)
    object_count = Column(Integer)

def register_models():
    """Register Models and create metadata.

    Called from vsm.db.sqlalchemy.__init__ as part of loading the driver,
    it will never need to be called explicitly elsewhere unless the
    connection is lost and needs to be reestablished.
    """
    from sqlalchemy import create_engine
    models = (Backup,
              Migration,
              Service,
              SMBackendConf,
              SMFlavors,
              SMHardware,
              Hardware,
              HardwareMetadata,
              HardwareTypeExtraSpecs,
              HardwareTypes,
              HardwareGlanceMetadata,
              )
    engine = create_engine(FLAGS.sql_connection, echo=False)
    for model in models:
        model.metadata.create_all(engine)

class Device(BASE, VsmBase):
    """This table store the information about device on host"""

    __tablename__ = 'devices'
    id = Column(Integer, primary_key=True)
    service_id = Column(Integer, ForeignKey('services.id'), nullable=False)
    service = relationship(Service,
                           backref=backref('device'),
                           foreign_keys=service_id,
                           primaryjoin='and_('
                           'Device.service_id == Service.id,'
                           'Device.deleted == False)')

    name = Column(String(length=255), nullable=False)
    path = Column(String(length=255), nullable=False)
    journal = Column(String(length=255), nullable=True)
    #total_capacity_gb = Column(Float(), nullable=False)
    #free_capacity_gb = Column(Float())
    total_capacity_kb = Column(BigInteger, default=0, nullable=False)
    used_capacity_kb = Column(BigInteger, default=0, nullable=False)
    avail_capacity_kb = Column(BigInteger, default=0, nullable=False)
    device_type = Column(String(length=255))
    interface_type = Column(String(length=255))
    fs_type = Column(String(length=255), default="xfs")
    mount_point = Column(String(length=255))
    state = Column(String(length=255), default="MISSING")
    journal_state = Column(String(length=255), default="MISSING")

class StorageGroup(BASE, VsmBase):
    """This table store the storage groups"""

    __tablename__ = 'storage_groups'
    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    storage_class = Column(String(length=255), nullable=False)
    friendly_name = Column(String(length=255), nullable=False)
    rule_id = Column(Integer, nullable=False)
    drive_extended_threshold = Column(Integer, default=0, nullable=False)
    status = Column(String(length=255), default="OUT", nullable=False)
    take_id = Column(Integer, nullable=True)
    take_order = Column(Integer, nullable=True)
    marker = Column(String(length=255), nullable=True)
    choose_num = Column(Integer, nullable=True)
    choose_type = Column(String(length=255), nullable=True)

class Zone(BASE, VsmBase):
    """This table store the zones"""

    __tablename__ = 'zones'
    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    parent_id = Column(Integer, ForeignKey('zones.id'), nullable=True)
    type = Column(Integer)

class OsdState(BASE, VsmBase):
    """This table maintains the information about osd."""

    __tablename__ = 'osd_states'
    id = Column(Integer, primary_key=True)
    service_id = Column(Integer, ForeignKey('services.id'), nullable=False)
    service = relationship(Service,
                           backref=backref('osd_state'),
                           lazy='subquery',
                           foreign_keys=service_id,
                           primaryjoin='and_('
                           'OsdState.service_id == Service.id,'
                           'OsdState.deleted == False)')
    zone_id = Column(Integer, ForeignKey('zones.id'), nullable=False)
    zone = relationship(Zone,
                            backref=backref('osd_state'),
                            foreign_keys=zone_id,
                            lazy='subquery',
                            primaryjoin='and_('
                                'OsdState.zone_id== Zone.id,'
                                'OsdState.deleted == False)')
    device_id = Column(Integer, ForeignKey('devices.id'), nullable=False)
    device = relationship(Device,
                          backref=backref('osd_state'),
                          lazy='subquery',
                          foreign_keys=device_id,
                          primaryjoin='and_('
                          'OsdState.device_id == Device.id,'
                          'OsdState.deleted == False)')
    storage_group_id = Column(Integer,
                              ForeignKey('storage_groups.id'),
                              nullable=False)
    storage_group = relationship(StorageGroup,
                                 backref=backref('osd_state'),
                                 lazy='subquery',
                                 foreign_keys=storage_group_id,
                                 primaryjoin='and_('
                                 'OsdState.storage_group_id == StorageGroup.id,'
                                 'OsdState.deleted == False)')

    osd_name = Column(String(length=255), nullable=False)
    cluster_id = Column(Integer)
    state = Column(String(length=255), default="down")
    public_ip = Column(String(length=255))
    cluster_ip = Column(String(length=255))
    weight = Column(Float)
    operation_status = Column(String(length=255))
    osd_location = Column(String(length=255))

class Recipe(BASE, VsmBase):
    """This table store the recipes."""

    __tablename__ = 'recipes'
    id = Column(Integer, primary_key=True, nullable=False)
    recipe_name = Column(String(length=255), unique=True, nullable=False)
    pg_num = Column(Integer, nullable=False)
    pgp_num = Column(Integer, nullable=False)
    size = Column(Integer, default=0, nullable=False)
    min_size = Column(Integer)
    crush_ruleset = Column(Integer, nullable=False)
    crash_replay_interval = Column(Integer)

class VsmCapacityManage(BASE, VsmBase):
    """ VSM user capacity management

    To manage the capacity that VSM user can use. The most important in
    this table is capacity_quota_mb and capacity_used_mb which denote
    the quota storage and used storage respectively.
    """

    __tablename__ = 'vsm_capacity_manages'
    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    capacity_quota_mb = Column(Integer)
    capacity_used_mb = Column(Integer)
    testyr = Column(Integer)

class Cluster(BASE, VsmBase):
    """This table store the clusters."""

    __tablename__ = 'clusters'
    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=True)
    file_system = Column(String(length=255), default='xfs', nullable=True)
    mkfs_option = Column(String(length=255), nullable=True)
    mount_option = Column(String(length=255), nullable=True)
    management_network = Column(String(length=255), nullable=True)
    ceph_public_network = Column(String(length=255), nullable=True)
    cluster_network = Column(String(length=255), nullable=True)
    # journal_size = Column(Integer, default=0)
    deleted_times = Column(Integer, default=0)
    info_dict = Column(Text, nullable=True)
    ceph_conf = Column(String(length=10485760), nullable=True)
    #osd_heartbeat_interval = Column(Integer, nullable=True)
    #osd_heartbeat_grace=Column(Integer, nullable=True)

class StoragePool(BASE, VsmBase):
    """This table store the storage pools."""

    __tablename__ = 'storage_pools'
    id = Column(Integer, primary_key=True, nullable=False)
    pool_id = Column(Integer, nullable=False)
    name = Column(String(length=255), nullable=False)
    status = Column(String(length=255), default="pending")
    recipe_id = Column(Integer, ForeignKey('recipes.id'), nullable=False)
    recipes = relationship(Recipe,
                            backref=backref('storage_pool'),
                            foreign_keys=recipe_id,
                            primaryjoin='and_('
                                'StoragePool.recipe_id == Recipe.id,'
                                'StoragePool.deleted == False)')
    cluster_id = Column(Integer, ForeignKey('clusters.id'), nullable=False)
    cluster = relationship(Cluster,
                           backref=backref('storage_pool'),
                           foreign_keys=cluster_id,
                           primaryjoin='and_('
                           'StoragePool.cluster_id == Cluster.id,'
                           'StoragePool.deleted == False)')
    pg_num = Column(Integer)
    pgp_num = Column(Integer)
    size = Column(Integer, default=0)
    min_size = Column(Integer)
    #TODO this crush_ruleset is diff from the rule_id in storage groups
    #crush_ruleset = Column(Integer, ForeignKey('storage_groups.rule_id'), nullable=False)
    #ruleset = relationship(StorageGroup,
    #                       backref=backref('storage_pool'),
    #                       foreign_keys=crush_ruleset,
    #                       primaryjoin='and_('
    #                       'StoragePool.crush_ruleset == StorageGroup.rule_id,'
    #                       'StoragePool.deleted == False)')
    crush_ruleset = Column(Integer, nullable=False)  
    primary_storage_group_id = Column(Integer, ForeignKey('storage_groups.id'), nullable=False)
    storage_group = relationship(StorageGroup,
                           backref=backref('storage_pool'),
                           lazy='subquery',
                           foreign_keys=primary_storage_group_id,
                           primaryjoin='and_('
                           'StoragePool.primary_storage_group_id == StorageGroup.id,'
                           'StoragePool.deleted == False)')
    crash_replay_interval = Column(Integer)
    created_by = Column(String(length=50), nullable=False)
    tag = Column(String(length=16), nullable=False)
    num_bytes = Column(Integer)
    num_objects = Column(Integer)
    num_object_clones = Column(Integer)
    num_objects_degraded = Column(Integer)
    num_objects_unfound = Column(Integer)
    num_read = Column(Integer)
    num_read_kb = Column(Integer)
    num_write = Column(Integer)
    num_write_kb = Column(Integer)
    read_bytes_sec = Column(Integer)
    write_bytes_sec = Column(Integer)
    op_per_sec = Column(Integer)
    ec_status = Column(String(length=255))
    cache_tier_status = Column(String(length=255))
    quota = Column(Integer, default=None)
    cache_mode = Column(String(length=255))
    replica_storage_group =Column(String(length=255))
    max_pg_num_per_osd = Column('max_pg_num_per_osd',Integer, nullable=False,default=100)
    auto_growth_pg = Column('auto_growth_pg',Integer, nullable=False,default=0)

class InitNode(BASE, VsmBase):
    """This table init nodes."""

    __tablename__ = 'init_nodes'
    id = Column(Integer, primary_key=True, nullable=False)
    host = Column(String(length=255), nullable=True)
    ceph_ver = Column(String(length=255), nullable=True)
    service_id = Column(Integer, ForeignKey('services.id'), nullable=False)
    service = relationship(Service,
                            backref=backref('init_node'),
                            foreign_keys=service_id,
                            primaryjoin='and_('
                                'InitNode.service_id == Service.id,'
                                'InitNode.deleted == False)')
    raw_ip = Column(String(length=255), nullable=True)
    primary_public_ip = Column(String(length=255), nullable=True)
    secondary_public_ip = Column(String(length=255), nullable=True)
    cluster_ip = Column(String(length=255), nullable=True)
    zone_id = Column(Integer, ForeignKey('zones.id'), nullable=True)
    zone = relationship(Zone,
                            backref=backref('init_node'),
                            foreign_keys=zone_id,
                            primaryjoin='and_('
                                'InitNode.zone_id== Zone.id,'
                                'InitNode.deleted == False)')

    mds = Column(String(length=255), nullable=True)
    type = Column(String(length=255), nullable=True)
    id_rsa_pub = Column(String(length=1024), nullable=True)

    cluster_id = Column(Integer,
                        ForeignKey('clusters.id'),
                        nullable=False)
    cluster = relationship(Cluster,
                            backref=backref('init_node'),
                            foreign_keys=cluster_id,
                            primaryjoin='and_('
                                'InitNode.cluster_id == Cluster.id,'
                                'InitNode.deleted == False)')
    data_drives_number = Column(String(length=255), nullable=True)
    status = Column(String(length=255), default='not available', nullable=True)
    pre_status = Column(String(length=255), nullable=True)

    def __repr__(self):
        return 'Host: %s' % self.host

class Vsmapp(BASE, VsmBase):
    """This tables contains the cluster info which will consume ceph."""
    __tablename__ = 'vsmapps'
    id = Column(Integer, primary_key=True, nullable=False)
    user_id = Column(Integer, nullable=False)
    project_id = Column(Integer, nullable=False)
    uuid = Column(String(length=255), nullable=False)
    display_name = Column(String(length=50), nullable=True)
    app_type = Column(String(length=50), default='OpenStack')
    storage_type = Column(String(length=50), default='rbd')
    status = Column(String(length=50), nullable=True)

class Appnode(BASE, VsmBase):
    """This table stores all the appnodes info."""
    __tablename__ = 'appnodes'

    id = Column(Integer, primary_key=True, nullable=False)
    # ip = Column(String(length=50), nullable=False)
    os_tenant_name = Column(String(length=50), nullable=False)
    os_username = Column(String(length=50), nullable=False)
    os_password = Column(String(length=50), nullable=False)
    os_auth_url = Column(String(length=255), nullable=False)
    os_region_name = Column(String(length=255), nullable=True)
    ssh_user = Column(String(length=255), nullable=False)
    uuid = Column(String(length=255), nullable=False)
    vsmapp_id = Column(Integer, ForeignKey(Vsmapp.id), nullable=False)
    ssh_status = Column(String(length=50), nullable=True)
    log_info = Column(Text, nullable=True)
    vsmapp = relationship(Vsmapp,
                          backref=backref('appnode'),
                          foreign_keys=vsmapp_id,
                          primaryjoin='and_('
                          'Appnode.vsmapp_id == Vsmapp.id,'
                          'Appnode.deleted == False)')

class StoragePoolUsage(BASE, VsmBase):
    """Storage pool usage """
    __tablename__ = 'storage_pool_usages'

    id = Column(Integer, primary_key=True, nullable=False)
    pool_id = Column(Integer,
                     ForeignKey(StoragePool.id),
                     nullable=False)
    vsmapp_id = Column(Integer,
                       ForeignKey(Vsmapp.id),
                       nullable=False)
    appnode_id = Column(Integer, nullable=False)
    cinder_volume_host = Column(String(length=255), nullable=True)
    attach_status = Column(String(length=255), nullable=False)
    pools = relationship(StoragePool,
                         backref=backref('storage_pool_usage'),
                         foreign_keys=pool_id,
                         primaryjoin='and_('
                         'StoragePoolUsage.pool_id == StoragePool.id,'
                         'StoragePoolUsage.deleted == False)')
    attach_at = Column(DateTime, default=timeutils.utcnow)
    terminate_at = Column(DateTime)

    vsmapps = relationship(Vsmapp,
                           backref=backref('storage_pool_usage'),
                           foreign_keys=vsmapp_id,
                           primaryjoin='and_('
                           'StoragePoolUsage.vsmapp_id == Vsmapp.id,'
                           'StoragePoolUsage.deleted == False)')
    as_glance_store_pool = Column(Boolean, default=False)


class Summary(BASE, VsmBase):
    """ ceph summary report """
    __tablename__ = 'summary'

    id = Column(Integer, primary_key=True, nullable=False)
    cluster_id = Column(Integer,
                        ForeignKey('clusters.id'),
                        nullable=False)
    summary_type = Column(String(length=50), nullable=False)
    summary_data = Column(Text, nullable=False)
    cluster = relationship(Cluster,
                           backref=backref('summary'),
                           foreign_keys=cluster_id,
                           primaryjoin='and_('
                                       'Summary.cluster_id == Cluster.id,'
                                       'Summary.deleted == False)')

class Monitor(BASE, VsmBase):
    """ ceph monitor stat data"""
    __tablename__ = 'monitors'

    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    address = Column(String(length=255))
    health = Column(String(length=255))
    details = Column(String(length=255))
    skew = Column(String(length=255))
    latency = Column(String(length=255))
    kb_total = Column(Integer)
    kb_used = Column(Integer)
    kb_avail = Column(Integer)
    avail_percent = Column(Integer)

class PlacementGroup(BASE, VsmBase):
    """ ceph placement group report """
    __tablename__ = 'placement_groups'

    id = Column(Integer, primary_key=True, nullable=False)
    pgid = Column(String(length=255), nullable=False)
    state = Column(String(length=255), nullable=False)
    up = Column(String(length=255), nullable=False)
    acting = Column(String(length=255), nullable=False)

class LicenseStatus(BASE, VsmBase):
    """License status """
    __tablename__ = 'license_status'

    id = Column(Integer, primary_key=True)
    license_accept = Column(Boolean, default=False)
    #FIXME(fengqian): Does it need to add user info here?
    #user_name = Column(String(length=50), nullable=False)

class RBD(BASE, VsmBase):
    """ ceph rbd report """
    __tablename__ = 'rbds'

    id = Column(Integer, primary_key=True, nullable=False)
    pool = Column(String(length=255), nullable=False)
    image = Column(String(length=255), nullable=False)
    size = Column(BigInteger, nullable=False)
    format = Column(Integer, nullable=False)
    objects = Column(Integer, nullable=False)
    order = Column(Integer, nullable=False)

class MDS(BASE, VsmBase):
    """ ceph MDS report """
    __tablename__ = 'mdses'

    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    gid = Column(Integer, nullable=False)
    state = Column(String(length=255), nullable=False)
    address = Column(String(length=255), nullable=False)

class VsmSettings(BASE, VsmBase):
    """ vsm UI settings and vsm settings model
    """
    __tablename__ = 'vsm_settings'

    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    value = Column(String(length=255), nullable=False)
    default_value = Column(String(length=255), nullable=False)

class ErasureCodeProfile(BASE, VsmBase):
    """erasure code profile  """
    __tablename__ = 'ec_profiles'

    id = Column(Integer, primary_key=True, nullable=False)
    name = Column(String(length=255), nullable=False)
    plugin = Column(String(length=255), nullable=False)
    plugin_path = Column(String(length=255), nullable=False)
    pg_num = Column(Integer, nullable=False)
    plugin_kv_pair = Column(Text, nullable=False)
    

class CephPerformanceMetric(BASE, VsmBase):
    """ ceph performance metric and value from diamond
    """
    __tablename__ = 'metrics'

    id = Column(Integer, primary_key=True, nullable=False)
    metric = Column(String(length=255), nullable=False)
    hostname = Column(String(length=255), nullable=False)
    instance = Column(String(length=255), nullable=False)
    value = Column(String(length=255), nullable=False)
    timestamp = Column(Integer, nullable=False)

class Config(BASE, VsmBase):
    """ configurable items for vsm, ceph, os, etc.
    """
    __tablename__ = 'config'

    id = Column(Integer, primary_key=True, nullable=False)
    name = Column('name', String(length=255), nullable=False)
    value = Column('value', String(length=255), nullable=True)
    default_value = Column('default_value', String(length=255), nullable=False)
    category = Column('category', String(length=255), nullable=True)
    section = Column('section', String(length=255), nullable=False)
    description = Column('description', String(length=255), nullable=True)
    alterable = Column('alterable', Boolean(create_constraint=True, name=None))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob
from webob import exc
from vsm import utils
from vsm.api import extensions
from vsm.api.openstack import wsgi
from vsm.agent import appnodes
from vsm.api import xmlutil
from vsm import flags
from vsm import conductor
from vsm import exception
from vsm.openstack.common import log as logging
from vsm.api.views import appnodes as appnodes_views
from vsm.openstack.common.gettextutils import _

"""App nodes API"""

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_appnode(elem, detailed=False):
    elem.set('id')
    elem.set('vsmapp_id')
    elem.set('ip')
    elem.set('ssh_status')
    elem.set('log_info')

    if detailed:
        elem.set('created_at')

vsmapp_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class AppnodeTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('appnode', selector='appnode')
        make_appnode(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=vsmapp_nsmap)

class AppnodesTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('appnodes')
        elem = xmlutil.SubTemplateElement(root, 'appnode', selector='appnodes')
        make_appnode(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=vsmapp_nsmap)

class AppnodesController(wsgi.Controller):
    """The vsm app nodes API controller."""
    _view_builder_class = appnodes_views.ViewBuilder

    def __init__(self):
        self.conductor_api = conductor.API()
        super(AppnodesController, self).__init__()

    @wsgi.serializers(xml=AppnodeTemplate)
    def show(self, req, id):
        """Get details info of an appnode."""
        context = req.environ['vsm.context']
        try:
            appnode = self.conductor_api.get_appnode(context, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()
        return {"appnode": appnode}

    @wsgi.serializers(xml=AppnodesTemplate)
    def index(self, req):
        """Returns a summary list of appnodes."""
        return self._get_nodes(req)

    def _get_nodes(self, req):
        """Returns a list of appnodes, transformed through view builder."""
        context = req.environ['vsm.context']
        nodes = appnodes.get_all_nodes(context)
        node_view = self._view_builder.index(nodes)
        return node_view

    def _get_appnodes_search_options(self):
        """Return appnodes search options allowed by non-admin."""
        return ('vsmapp_id',)

    @wsgi.response(202)
    def create(self, req, body):
        """create app nodes."""
        LOG.debug(_('Creating new app nodes %s'), body)

        if not self.is_valid_body(body, 'appnodes'):
            raise exc.HTTPBadRequest()

        context = req.environ['vsm.context']
        auth_openstack = body['appnodes']
        # if not isinstance(ip_list, list):
        #     expl = _('Invalid Request body: should be a list of IP address.')
        #     raise webob.exc.HTTPBadRequest(explanation=expl)
        node_list = appnodes.create(context, auth_openstack)
        node_view = self._view_builder.index(node_list)
        # for node in node_list:
            #LOG.info('Node %s, Node id: %s, Node ip: %s' % (node, node.id, node.ip))
            #ssh = utils.SSHClient(ip='10.239.131.170', user='root', key_file=r'~/.ssh/id_rsa')
            #ssh = utils.SSHClient(node.ip, 'root', '~/.ssh/id_rsa')
            #ret = ssh.check_ssh(retries=1)
            #status = 'reachable' if ret else 'unreachable'
            # status = 'reachable'
            # appnodes.update(context, node.id, status)

        return webob.Response(status_int=201)

    @wsgi.serializers(xml=AppnodesTemplate)
    def delete(self, req, id):
        """delete app node by id."""
        LOG.debug(_('Removing app node by id %s' % id))
        context = req.environ['vsm.context']
        if id is None:
            expl = _('Request body and URI mismatch')
            raise webob.exc.HTTPBadRequest(explanation=expl)
        id = str(id)
        appnodes.destroy(context, id)
        return webob.Response(status_int=201)

    @wsgi.serializers(xml=AppnodesTemplate)
    def update(self, req, id, body):
        LOG.debug(_('Updating app node by id %s' % id))
        context = req.environ['vsm.context']

        if not self.is_valid_body(body, 'appnode'):
            raise exc.HTTPBadRequest()

        body = body.get('appnode')
        LOG.debug('PUT body: %s' % body)

        if id is None:
            expl = _('Request body and URI mismatch: No appnode_id')
            raise webob.exc.HTTPBadRequest(explanation=expl)
        # convert from unicode to str
        id = str(id)
        os_tenant_name = body.get('os_tenant_name', None)
        os_username = body.get('os_username', None)
        os_password = body.get('os_password', None)
        os_auth_url = body.get('os_auth_url', None)

        if not os_tenant_name or not os_username or not os_password or not os_auth_url:
            expl = _('Request body and URI mismatch: os_tenant_name or os_username or os_password or os_auth_url required.')
            raise webob.exc.HTTPBadRequest(explanation=expl)

        appnodes.update(context, id, body)
        return webob.Response(status_int=201)

def remove_invalid_options(context, search_options, allowed_search_options):
    """Remove search options that are not valid for non-admin API/context."""
    if context.is_admin:
        # Allow all options
        return
    # Otherwise, strip out all unknown options
    unknown_options = [opt for opt in search_options
                       if opt not in allowed_search_options]
    bad_options = ", ".join(unknown_options)
    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
    LOG.debug(log_msg)
    for opt in unknown_options:
        del search_options[opt]

class Appnodes(extensions.ExtensionDescriptor):
    """Appnodes support."""

    name = 'Appnodes'
    alias = 'appnodes'
    namespace = 'http://docs.openstack.org/storage/ext/appnodes/api/v1'
    updated = '2014-03-12T00:00:00+00:00'

    def get_resources(self):
        resources = []
        res = extensions.ResourceExtension(Appnodes.alias, AppnodesController())

        resources.append(res)
        return resources



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import rbd_pools as rbd_pool_views
from vsm import conductor
from vsm import scheduler
from vsm import exception

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_rbd_pool(elem, detailed=False):
    elem.set('id')
    elem.set('name')
    elem.set('address')
    elem.set('health')
    elem.set('detail')
    elem.set('skew')
    elem.set('latency')
    elem.set('kb_total')
    elem.set('kb_used')
    elem.set('kb_avail')
    elem.set('percent_avail')

    if detailed:
        pass

rbd_pool_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class RBDPoolTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('rbd_pool', selector='rbd_pool')
        make_rbd_pool(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=rbd_pool_nsmap)

class RBDPoolsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('rbd_pools')
        elem = xmlutil.SubTemplateElement(root, 'rbd_pool', selector='rbd_poolss')
        make_rbd_pool(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=rbd_pool_nsmap)

class Controller(wsgi.Controller):
    """The rbd_pools API controller for the OpenStack API."""
    _view_builder_class = rbd_pool_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        super(Controller, self).__init__()

    #def _get_zone_search_options(self):
    #    """Return zone search options allowed by non-admin."""
    #    return ('id', 'name', 'public_ip')

    def _get_rbd_pool(self, context, req, id):
        """Utility function for looking up an instance by id."""
        try:
            rbd_pool = self.conductor_api.rbd_pool_get(context, id)
        except exception.NotFound:
            msg = _("rbd_pool could not be found")
            raise exc.HTTPNotFound(explanation=msg)
        return rbd_pool

    @wsgi.serializers(xml=RBDPoolsTemplate)
    def show(self, req, id):
        """Return data about the given rbd_pool."""
        context = req.environ['vsm.context']

        try:
            rbd_pool = self._get_rbd_pool(context, req, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return {'rbd_pool': rbd_pool}

    @wsgi.serializers(xml=RBDPoolsTemplate)
    def index(self, req):
        """Return a list of rbd pools."""

        context = req.environ['vsm.context']
        limit = req.GET.get('limit', None)
        marker = req.GET.get('marker', None)
        sort_keys = req.GET.get('sort_keys', None)
        sort_dir = req.GET.get('sort_dir', None)

        rbd_pools = self.conductor_api.rbd_get_all(context,
                                                   limit=limit,
                                                   marker=marker,
                                                   sort_keys=sort_keys,
                                                   sort_dir=sort_dir)
        LOG.info('vsm/api/v1/rbd_pool.py rbd_pools:%s' % rbd_pools)

        return self._view_builder.index(req, rbd_pools)

    @wsgi.serializers(xml=RBDPoolsTemplate)
    def detail(self, req):

        """Get rbd_pool list."""
        #search_opts = {}
        #search_opts.update(req.GET)
        context = req.environ['vsm.context']
        #remove_invalid_options(context,
        #                        search_opts,
        #                        self._get_zone_search_options)
        #zones = self.conductor_api.get_zone_list(context)
        limit = req.GET.get('limit', None)
        marker = req.GET.get('marker', None)
        sort_keys = req.GET.get('sort_keys', None)
        sort_dir = req.GET.get('sort_dir', None)

        rbd_pools = self.conductor_api.rbd_get_all(context, limit, marker,
						   sort_keys, sort_dir)
        #LOG.info('vsm/api/v1/rbd_pools.py detailed rbd_pools:%s' % rbd_pools)

        return self._view_builder.detail(req, rbd_pools)

    def summary(self, req, body=None):
        LOG.info('CEPH_LOG rbd_pool-summary body %s ' % body)
        context = req.environ['vsm.context']
        return {'rbd-summary':{'epoch': 123,
                               'num_rbd_pools': 12,
                               'num_up_rbd_pools': 8,
                               'num_in_rbd_pools': 8,
                               'nearfull': False,
                               'full': False,
                               }}

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))

#def remove_invalid_options(context, search_options, allowed_search_options):
#    """Remove search options that are not valid for non-admin API/context."""
#    if context.is_admin:
#        # Allow all options
#        return
#    # Otherwise, strip out all unknown options
#    unknown_options = [opt for opt in search_options
#                       if opt not in allowed_search_options]
#    bad_options = ", ".join(unknown_options)
#    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
#    LOG.debug(log_msg)
#    for opt in unknown_options:
#        del search_options[opt]



#    Copyright 2014 Intel
#    All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Handles all requests to the physical service."""

from oslo.config import cfg
from vsm.db import base
from vsm.physical import rpcapi
from vsm.openstack.common import log as logging

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

class API(base.Base):
    """Physical API that does updates via RPC to the PhysicalManager."""

    def __init__(self, **kwargs):
        self.physical_rpcapi = rpcapi.PhysicalAPI()
        super(API, self).__init__(**kwargs)

    def create(self, context, physical_info):
        """Use physical info to create a physical node."""
        return self.db.init_node_create(context, physical_info)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""The storage_pools api."""

import uuid
import webob
from webob import exc
import re

from vsm.api import common
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import exception
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import pools as pools_views
from vsm.openstack.common import jsonutils
from vsm import db
from vsm import utils
from vsm import conductor
from vsm import scheduler

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def _translate_attachment_detail_view(_context, vol):
    """Maps keys for attachment details view."""

    d = _translate_attachment_summary_view(_context, vol)

    # No additional data / lookups at the moment

    return d

def _translate_attachment_summary_view(_context, vol):
    """Maps keys for attachment summary view."""
    d = {}

    storage_pool_id = vol['id']

# NOTE(justinsb): We use the storage_pool id as the id of the attachment object
    d['id'] = storage_pool_id

    d['storage_pool_id'] = storage_pool_id
    d['server_id'] = vol['instance_uuid']
    if vol.get('mountpoint'):
        d['device'] = vol['mountpoint']

    return d

def _translate_storage_pool_detail_view(context, vol, image_id=None):
    """Maps keys for storage_pools details view."""

    d = _translate_storage_pool_summary_view(context, vol, image_id)

    # No additional data / lookups at the moment

    return d

def _translate_storage_pool_summary_view(context, vol, image_id=None):
    """Maps keys for storage_pools summary view."""
    d = {}

    d['id'] = vol['id']
    d['status'] = vol['status']
    d['size'] = vol['size']
    d['availability_zone'] = vol['availability_zone']
    d['created_at'] = vol['created_at']

    d['attachments'] = []
    if vol['attach_status'] == 'attached':
        attachment = _translate_attachment_detail_view(context, vol)
        d['attachments'].append(attachment)

    d['display_name'] = vol['display_name']
    d['display_description'] = vol['display_description']

    if vol['storage_pool_type_id'] and vol.get('storage_pool_type'):
        d['storage_pool_type'] = vol['storage_pool_type']['name']
    else:
        # TODO(bcwaldon): remove str cast once we use uuids
        d['storage_pool_type'] = str(vol['storage_pool_type_id'])

    d['snapshot_id'] = vol['snapshot_id']
    d['source_volid'] = vol['source_volid']

    if image_id:
        d['image_id'] = image_id

    LOG.audit(_("vol=%s"), vol, context=context)

    if vol.get('storage_pool_metadata'):
        metadata = vol.get('storage_pool_metadata')
        d['metadata'] = dict((item['key'], item['value']) for item in metadata)
    # avoid circular ref when vol is a StoragePool instance
    elif vol.get('metadata') and isinstance(vol.get('metadata'), dict):
        d['metadata'] = vol['metadata']
    else:
        d['metadata'] = {}

    if vol.get('storage_pool_glance_metadata'):
        d['bootable'] = 'true'
    else:
        d['bootable'] = 'false'

    return d

def make_attachment(elem):
    elem.set('id')
    elem.set('server_id')
    elem.set('storage_pool_id')
    elem.set('device')

def make_storage_pool(elem):
    elem.set('id')
    elem.set('status')
    elem.set('size')
    elem.set('availability_zone')
    elem.set('created_at')
    elem.set('display_name')
    elem.set('display_description')
    elem.set('storage_pool_type')
    elem.set('snapshot_id')
    elem.set('source_volid')

    attachments = xmlutil.SubTemplateElement(elem, 'attachments')
    attachment = xmlutil.SubTemplateElement(attachments, 'attachment',
                                            selector='attachments')
    make_attachment(attachment)

    # Attach metadata node
    elem.append(common.MetadataTemplate())

storage_pool_nsmap = {None: xmlutil.XMLNS_VOLUME_V1, 'atom': xmlutil.XMLNS_ATOM}

class StoragePoolTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_pool', selector='storage_pool')
        make_storage_pool(root)
        return xmlutil.MasterTemplate(root, 1, nsmap=storage_pool_nsmap)

class StoragePoolsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_pools')
        elem = xmlutil.SubTemplateElement(root,
                                            'storage_pool',
                                            selector='storage_pools')
        make_storage_pool(elem)
        return xmlutil.MasterTemplate(root, 1, nsmap=storage_pool_nsmap)

class CommonDeserializer(wsgi.MetadataXMLDeserializer):
    """Common deserializer to handle xml-formatted storage_pool requests.

       Handles standard storage_pool attributes as well as the optional metadata
       attribute
    """

    metadata_deserializer = common.MetadataXMLDeserializer()

    def _extract_storage_pool(self, node):
        """Marshal the storage_pool attribute of a parsed request."""
        storage_pool = {}
        storage_pool_node = self.find_first_child_named(node, 'storage_pool')

        attributes = ['display_name', 'display_description', 'size',
                      'storage_pool_type', 'availability_zone']
        for attr in attributes:
            if storage_pool_node.getAttribute(attr):
                storage_pool[attr] = storage_pool_node.getAttribute(attr)

        metadata_node = self.find_first_child_named(storage_pool_node,
                                                    'metadata')
        if metadata_node is not None:
            storage_pool['metadata'] = self.extract_metadata(metadata_node)

        return storage_pool

class CreateDeserializer(CommonDeserializer):
    """Deserializer to handle xml-formatted create storage_pool requests.lder
       handles standard storage_pool attributes as well as the optional metadata
       attribute
    """

    def default(self, string):
        """Deserialize an xml-formatted storage_pool create request."""
        dom = utils.safe_minidom_parse_string(string)
        storage_pool = self._extract_storage_pool(dom)
        return {'body': {'storage_pool': storage_pool}}

class StoragePoolController(wsgi.Controller):
    """The StoragePools API controller for the OpenStack API."""
    _view_builder_class = pools_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        
        # at this point, we will add the map as an data member 
        # about storage_group and pool_size
        #self.group_map = scheduler_api.

        super(StoragePoolController, self).__init__()

    @wsgi.serializers(xml=StoragePoolTemplate)
    def show(self, req, id):
        """ Return data about the given storage pool """

        context = req.environ['vsm.context']

        try:
            storage_pool = self.conductor_api.get_storage_pool(context, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return {"pool": storage_pool}

    def delete(self, req, id):
        """Delete a storage_pool."""
        context = req.environ['vsm.context']

        LOG.audit(_("Delete storage_pool with id: %s"), id, context=context)

        try:
            storage_pool = self.conductor_api.get(context, id)
            self.conductor_api.delete(context, storage_pool)
        except exception.NotFound:
            raise exc.HTTPNotFound()
        return webob.Response(status_int=202)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def index(self, req):
        """Returns a summary list of storage_pools."""
        return self._items(req,
                            entity_maker=_translate_storage_pool_summary_view)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def test_scheduler(self, req, body=None):
        """Returns a detailed list of host status."""
        body_info = body.get('request', None)
        search_opts = {}
        search_opts.update(req.GET)

        context = req.environ['vsm.context']
        remove_invalid_options(context,
                                search_opts, 
                                self._get_storage_pool_search_options)

        res = self.scheduler_api.test_service(context, body_info)
        return {'key': res}

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def create(self, req, body=None):
        """Create a storage pool."""
        LOG.info(body)

        #{'pool':
        #    {'replicationFactor': 3,
        #    'name': 'test',
        #    'enablePoolQuota': False,
        #    'storageGroupId': '1',
        #    'u'replicatedStorageGroupId': '1',
        #    'clusterId': '0',
        #    'tag': 'abc',
        #    'createdBy': 'VSM',
        #    'ecProfileId': '1',
        #    'ecFailureDomain': 'osd',
        #    'poolQuota': 0
        #    }
        #}

        if not self.is_valid_body(body, 'pool'):
            raise exc.HTTPUnprocessableEntity()

        context = req.environ['vsm.context']
        pool_dict = body['pool']

        for key in ('name', 'createdBy', 'storageGroupName'):
            if not key in pool_dict:
                msg = _("%s is not defined in pool" % key)
                raise exc.HTTPBadRequest(explanation=msg)

        name = pool_dict['name'].strip()
        created_by = pool_dict['createdBy'].strip()

        storage_group_name = pool_dict['storageGroupName']
        tag = pool_dict['tag'].strip()
        cluster_id = pool_dict['clusterId']

        try:
            cluster_id = int(str(cluster_id))
        except ValueError:
            msg = _('cluster_id must be an interger value')
            raise exc.HTTPBadRequest(explanation=msg)

        storage_group = db.storage_group_get_by_name(context, storage_group_name)
        rule_id = storage_group['rule_id']
        storage_group_id =  storage_group['id']
        size = db.get_size_by_storage_group_name(context,storage_group_name)
        size = int(size)
        if size == 0:
            pool_default_size = db.vsm_settings_get_by_name(context,'osd_pool_default_size')
            size = int(pool_default_size.value)
        #LOG.info('size=====%s'%size)
        #osd_num = 2 #TODO self.scheduler_api.get_osd_num_from_crushmap_by_rule(context, rule_id)
        is_ec_pool = pool_dict.get('ecProfileId')
        if is_ec_pool:
            #erasure code pool 
            body_info = {'name': name,
                        'cluster_id':cluster_id,
                        'storage_group_id':storage_group_id,
                        'storage_group_name':storage_group_name,
                        'ec_profile_id':pool_dict['ecProfileId'],
                        'ec_ruleset_root':storage_group['name'],
                        'ec_failure_domain':pool_dict['ecFailureDomain'],
                        'created_by':created_by,
                        'tag':tag}
             
        else:
            #replicated pool 
            crush_ruleset = rule_id#self.conductor_api.get_ruleset_id(context, storage_group_id)
            if crush_ruleset < 0:
                msg = _('crush_ruleset must be a non-negative integer value')
                raise exc.HTTPBadRequest(explanation=msg)

            #size = pool_dict['replicationFactor']
            #replica_storage_group_id = pool_dict['replicatedStorageGroupId']
            #try:
            #     size = int(str(size))
            #     if size < 1:
            #         msg = _('size must be > 1')
            #         raise exc.HTTPBadRequest(explanation=msg)
            #
            #     host_num = self.conductor_api.count_hosts_by_storage_group_id(context, storage_group_id)
            #     LOG.info("storage_group_id:%s,host_num:%s", storage_group_id, host_num)
            #     if size > host_num:
            #         msg = "The replication factor must be less than or equal to the number of storage nodes in the specific storage group in cluster!"
            #         return {'message': msg}
            # except ValueError:
            #     msg = _('size must be an interger value')
            #     raise exc.HTTPBadRequest(explanation=msg)

            #pg_num = self._compute_pg_num(context, osd_num, size)

            #vsm_id = str(uuid.uuid1()).split('-')[0]
            pg_num = pool_dict.get('pg_num', 64)
            #self._compute_pg_num(context, osd_num, size)
            body_info = {'name': name, #+ "-vsm" + vsm_id,
                        'cluster_id':cluster_id,
                        'storage_group_id':storage_group_id,
                        'storage_group_name':storage_group_name,
                        'pool_type':'replicated',
                        'crush_ruleset':crush_ruleset,
                        'pg_num':pg_num,
                        'pgp_num':pg_num,
                        'size':size,
                        'min_size':size,
                        'created_by':created_by,
                        'tag':tag}

        body_info.update({
            "quota": pool_dict.get("poolQuota"),
            "enable_quota": pool_dict.get("enablePoolQuota"),
            "max_pg_num_per_osd": pool_dict.get("max_pg_num_per_osd") or 100,
            "auto_growth_pg": pool_dict.get("auto_growth_pg") or 0,
        })
        #LOG.info('body_info=====%s'%body_info)
        return self.scheduler_api.create_storage_pool(context, body_info)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def add_cache_tier(self, req, body=None):
        """add a cache tier."""
        LOG.info(body)

        #{'cache_tier':
        #    {'storage_pool_id': 3,
        #    'cache_pool_id': 4,
        #    'cache_mode': 'writeback',
        #    }
        #}

        if not self.is_valid_body(body, 'cache_tier'):
            raise exc.HTTPUnprocessableEntity()

        context = req.environ['vsm.context']
        cache_tier_body = body['cache_tier']
        return self.scheduler_api.add_cache_tier(context, cache_tier_body)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def remove_cache_tier(self, req, body=None):
        """add a cache tier."""
        LOG.info(body)

        #{'cache_tier':
        #    {
        #    'cache_pool_id': 4,
        #    }
        #}

        if not self.is_valid_body(body, 'cache_tier'):
            raise exc.HTTPUnprocessableEntity()

        context = req.environ['vsm.context']
        cache_tier_body = body['cache_tier']
        return self.scheduler_api.remove_cache_tier(context, cache_tier_body)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def get_storage_group_list(self, req):
        """storage_group_list."""
        context = req.environ['vsm.context']
        storage_group_list = self.scheduler_api.get_storage_group_list(context)
        return storage_group_list

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def get_ec_profile_list(self, req):
        """ec_profile_list."""
        context = req.environ['vsm.context']
        ec_profile_list = db.ec_profile_get_all(context)
        ec_profiles = {"ec_profiles": [{"id": x.id, "name": x.name} for x in ec_profile_list]}
        return ec_profiles

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def list_storage_pool(self, req):
        """List storage pools."""
        """TODO: the func is deprecated. plz use detail()."""
        search_opts = {}
        search_opts.update(req.GET)
        context = req.environ['vsm.context']
        remove_invalid_options(context,
                                search_opts,
                                self._get_storage_pool_search_options)

        pools = self.conductor_api.list_storage_pool(context)
        pool_list = pools.values()
        sorted_pools = sorted(pool_list,
                              key=lambda item: item['id'])
        LOG.info('vsm/api/v1/storage_pool.py list_storage_pool')
        #mapping = self.conductor_api.get_mapping(context)

        #LOG.info("api:sorted_pools:%s" % sorted_pools)

        return self._view_builder.detail(sorted_pools)

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def resource_info(self, req, body=None):
        """Returns a detailed list of host status."""
        body_info = body.get('request', None)
        search_opts = {}
        search_opts.update(req.GET)

        context = req.environ['vsm.context']
        remove_invalid_options(context,
                                search_opts,
                                self._get_storage_pool_search_options)

        res = self.conductor_api.resource_info(context)
        LOG.info(' API return value')
        LOG.info(res)
        return {'resource_info': res}

    @wsgi.serializers(xml=StoragePoolsTemplate)
    def detail(self, req):
        """Returns a detailed list of storage_pools."""

        """List storage pools."""
        search_opts = {}
        search_opts.update(req.GET)
        context = req.environ['vsm.context']
        remove_invalid_options(context,
                                search_opts,
                                self._get_storage_pool_search_options)

        pools = self.conductor_api.list_storage_pool(context)
        pool_list = pools.values()
        sorted_pools = sorted(pool_list,
                              key=lambda item: item['id'])
        LOG.info('vsm/api/v1/storage_pool.py list_storage_pool')
        #mapping = self.conductor_api.get_mapping(context)

        return self._view_builder.detail(sorted_pools)
        #
        #LOG.debug(' comes to detail')
        #return self._items(req,
        #                    entity_maker=_translate_storage_pool_detail_view)

    def _compute_pg_num(self, context, osd_num, replication_num):
        """compute pg_num"""
        try:
            pg_count_factor = 200
            settings = db.vsm_settings_get_all(context)
            for setting in settings:
                if setting['name'] == 'pg_count_factor':
                    pg_count_factor = int(setting['value'])
            
            pg_num = pg_count_factor * osd_num//replication_num
        except ZeroDivisionError,e:
            raise exc.HTTPBadRequest(explanation=str(e))
        if pg_num < 1:
            msg = _("Could not compute proper pg_num.")
            raise exc.HTTPBadRequest(explanation=msg)
        return pg_num

    def _items(self, req, entity_maker):
        """Returns a list of storage_pools, transformed through entity_maker."""

        LOG.info(req)
        search_opts = {}
        search_opts.update(req.GET)

        context = req.environ['vsm.context']
        remove_invalid_options(context,
                                search_opts,
                                self._get_storage_pool_search_options())

        res = self.conductor_api.test_service(context)
        return {'pool': res}

    def _get_storage_pool_search_options(self):
        """Return storage_pool search options allowed by non-admin."""
        return ('display_name', 'status')

    @wsgi.serializers(xml=StoragePoolTemplate)
    def update(self, req, id, body):
        """Update a storage_pool."""
        context = req.environ['vsm.context']

        if not body:
            raise exc.HTTPUnprocessableEntity()

        if 'storage_pool' not in body:
            raise exc.HTTPUnprocessableEntity()

        storage_pool = body['storage_pool']
        update_dict = {}

        valid_update_keys = (
            'display_name',
            'display_description',
            'metadata',
        )

        for key in valid_update_keys:
            if key in storage_pool:
                update_dict[key] = storage_pool[key]

        try:
            storage_pool = self.conductor_api.get(context, id)
            self.conductor_api.update(context, storage_pool, update_dict)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        storage_pool.update(update_dict)

        return {'storage_pool':
                _translate_storage_pool_detail_view(context, 
                                                    storage_pool)}

def create_resource(ext_mgr):
    return wsgi.Resource(StoragePoolController(ext_mgr))

def remove_invalid_options(context, search_options, allowed_search_options):
    """Remove search options that are not valid for non-admin API/context."""
    if context.is_admin:
        # Allow all options
        return
    # Otherwise, strip out all unknown options
    unknown_options = [opt for opt in search_options
                       if opt not in allowed_search_options]
    bad_options = ", ".join(unknown_options)
    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
    LOG.debug(log_msg)
    for opt in unknown_options:
        del search_options[opt]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import storage_groups as storage_group_views
from vsm import conductor
from vsm import scheduler
from vsm import exception
from vsm import db

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_storage_group(elem, detailed=False):
    elem.set('id')
    elem.set('name')
    elem.set('address')
    elem.set('health')
    elem.set('detail')
    elem.set('skew')
    elem.set('latency')
    elem.set('kb_total')
    elem.set('kb_used')
    elem.set('kb_avail')
    elem.set('percent_avail')

    if detailed:
        pass

storage_group_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class StorageGroupTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_group', selector='storage_group')
        make_storage_group(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=storage_group_nsmap)

class StorageGroupsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_groups')
        elem = xmlutil.SubTemplateElement(root, 'storage_group', selector='storage_groupss')
        make_storage_group(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=storage_group_nsmap)

class Controller(wsgi.Controller):
    """The storage_groups API controller for the OpenStack API."""
    _view_builder_class = storage_group_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        super(Controller, self).__init__()

    #def _get_zone_search_options(self):
    #    """Return zone search options allowed by non-admin."""
    #    return ('id', 'name', 'public_ip')

    def _get_storage_group(self, context, req, id):
        """Utility function for looking up an instance by id."""
        try:
            storage_group = self.conductor_api.storage_group_get(context, id)
        except exception.NotFound:
            msg = _("storage_group could not be found")
            raise exc.HTTPNotFound(explanation=msg)
        return storage_group

    @wsgi.serializers(xml=StorageGroupTemplate)
    def create(self, req, body):
        """create zone."""
        LOG.info("CEPH_LOG storage group create body: %s" % body)
        context = req.environ['vsm.context']
        storage_group = body['storage_group']
        storage_group.update({"rule_id": -1})
        db.create_storage_group(context, storage_group)
        #self.scheduler_api.add_new_zone(context, body)
        return webob.Response(status_int=202)

    def create_with_takes(self, req, body=None):
        '''

        :param res:
        :param body:
        :return:
        '''
        LOG.info("CEPH_LOG create_with_takes body=%s"%body )
        context = req.environ['vsm.context']
        ret = self.scheduler_api.add_storage_group_to_crushmap_and_db(context,body)
        LOG.info('CEPH_LOG create_with_takes  ret=%s'%ret)
        return ret

    def update_with_takes(self, req, body=None):
        '''

        :param res:
        :param body:
        :return:
        '''
        LOG.info("CEPH_LOG update_with_takes body=%s"%body )
        context = req.environ['vsm.context']
        ret = self.scheduler_api.update_storage_group_to_crushmap_and_db(context,body)
        LOG.info('CEPH_LOG update_with_takes  ret=%s'%ret)
        return ret

    @wsgi.serializers(xml=StorageGroupsTemplate)
    def show(self, req, id):
        """Return data about the given storage_group."""
        context = req.environ['vsm.context']

        try:
            storage_group = self._get_storage_group(context, req, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return {'storage_group': storage_group}

    @wsgi.serializers(xml=StorageGroupsTemplate)
    def index(self, req):
        """Get storage_group list."""
        #search_opts = {}
        #search_opts.update(req.GET)
        context = req.environ['vsm.context']
        #remove_invalid_options(context,
        #                        search_opts,
        #                        self._get_zone_search_options)
        #zones = self.conductor_api.get_zone_list(context)
        storage_groups = [{}, {}]
        LOG.info('vsm/api/v1/storage_group.py storage_groups:%s' % storage_groups)

        return self._view_builder.index(req, storage_groups)

    @wsgi.serializers(xml=StorageGroupsTemplate)
    def detail(self, req):

        """Get storage_group list."""
        context = req.environ['vsm.context']
        storage_groups_db = self.conductor_api.storage_group_get_all(context)
        storage_groups = {}
        storage_group_name_list = []
        for storage_group_db in storage_groups_db:
            take_dict = {
                "id":storage_group_db['take_id'],
                "name":"",
                "order":storage_group_db['take_order'],
                "choose_type":storage_group_db['choose_type'],
                "choose_num":storage_group_db['choose_num']
            }
            if storage_group_db['take_id']:
                try:
                    take_dict['name'] = db.zone_get(context,storage_group_db['take_id'])['name']
                except:
                    self.scheduler_api.update_zones_from_crushmap_to_db(context)
                    take_dict['name'] = db.zone_get(context,storage_group_db['take_id'])['name']

            if storage_group_db['name'] in storage_group_name_list:
                storage_groups[storage_group_db['name']]['take_list'].append(take_dict)
            else:
                storage_group_name_list.append(storage_group_db['name'])
                storage_group_dict = {
                    'id':storage_group_db['id'],
                    'name':storage_group_db['name'],
                    'storage_class':storage_group_db['storage_class'],
                    'friendly_name':storage_group_db['friendly_name'],
                    'marker':storage_group_db['marker'],
                    'rule_id':storage_group_db['rule_id'],
                    'take_list':[take_dict],
                    'status': storage_group_db["status"],
                }
                storage_groups[storage_group_db['name']] = storage_group_dict

        storage_groups = storage_groups.values()

        LOG.info('vsm/api/v1/storage_groups.py detailed storage_groups:%s' % storage_groups)
        osds = self.conductor_api.osd_state_get_all(context)
        LOG.info('vsm/api/v1/storage_groups.py detailed osds:%s' % osds)
        sp_usages = db.get_storage_pool_usage(context)
        LOG.info('vsm/api/v1/storage_groups.py detailed sp usage:%s' % sp_usages)
        pools = self.conductor_api.list_storage_pool(context)
        LOG.info('vsm/api/v1/storage_groups.py detailed pools:%s' % pools)
        LOG.info('vsm/api/v1/storage_groups.py detailed pools:%s' % type(pools))
        for sp_usage in sp_usages:
            pool_id = str(sp_usage['pool_id'])
            if pool_id in pools:
                pools[pool_id]['attach_status'] = sp_usage['attach_status']

        rules = [storage_group['name'] for storage_group in storage_groups]
        rules_dict = {'rules':list(set(rules))}
        rule_osds = self.scheduler_api.get_osds_by_rules(context,rules_dict )
        for storage_group in storage_groups:
            osds_in_storage_group = rule_osds.get(storage_group['name'])#osd['storage_group']['id'] == storage_group["id"]]
            osd_cnt = len(osds_in_storage_group)
            storage_group['capacity_total'] = sum([osd["device"]['total_capacity_kb'] for osd in osds
                                               if osd['osd_name'] in osds_in_storage_group])
            storage_group['capacity_used'] = sum([osd["device"]['used_capacity_kb'] for osd in osds
                                               if osd['osd_name'] in osds_in_storage_group])
            storage_group['capacity_avail'] = sum([osd["device"]['avail_capacity_kb'] for osd in osds
                                               if osd['osd_name'] in osds_in_storage_group])

            nodes = {}
            #osd_cnt = 0
            for osd in osds:
                if not osd['storage_group']['id'] == storage_group["id"] or osd['state'] == FLAGS.vsm_status_uninitialized:
                    continue
                #osd_cnt = osd_cnt + 1
                k = osd['service']['host']
                if k not in osd:
                    nodes.setdefault(k, 0)
                nodes[k] += osd["device"]['used_capacity_kb']
            if not nodes:
                storage_group['largest_node_capacity_used'] = 0
            else:
                storage_group['largest_node_capacity_used'] = max(nodes.values())

            storage_group["attached_osds"] = osd_cnt
            # attached pools
            storage_group["attached_pools"] = len([pool for pool in pools.values()
                                                    if (pool['primary_storage_group_id'] == storage_group["id"])])
            try:
                storage_group['updated_at'] = osds[0]['updated_at']
            except:
                pass
            LOG.info("vsm/api/v1/storage_groups.py %s" % storage_group)

        return self._view_builder.index(req, storage_groups)

    def summary(self, req, body=None):
        LOG.info('CEPH_LOG storage_group-summary body %s ' % body)
        context = req.environ['vsm.context']
        return {'storage_group-summary':{'epoch': 123,
                               'num_storage_groups': 12,
                               'num_up_storage_groups': 8,
                               'num_in_storage_groups': 8,
                               'nearfull': False,
                               'full': False,
                               }}

    def get_default_pg_num(self, req):
        context = req.environ['vsm.context']
        storage_group_name= req.GET.get('storage_group_name',None)
        LOG.info('CEPH_LOG get_default_pg_num %s ' % storage_group_name)
        body = {'storage_group_name':storage_group_name}
        default_pg_num = self.scheduler_api.get_default_pg_num_by_storage_group(context,body)
        return default_pg_num

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))

#def remove_invalid_options(context, search_options, allowed_search_options):
#    """Remove search options that are not valid for non-admin API/context."""
#    if context.is_admin:
#        # Allow all options
#        return
#    # Otherwise, strip out all unknown options
#    unknown_options = [opt for opt in search_options
#                       if opt not in allowed_search_options]
#    bad_options = ", ".join(unknown_options)
#    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
#    LOG.debug(log_msg)
#    for opt in unknown_options:
#        del search_options[opt]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common

class ViewBuilder(common.ViewBuilder):

    def show(self, request, storage_type, brief=False):
        """Trim away extraneous storage type attributes."""
        trimmed = dict(id=storage_type.get('id'),
                       name=storage_type.get('name'),
                       extra_specs=storage_type.get('extra_specs'))
        return trimmed if brief else dict(storage_type=trimmed)

    def index(self, request, storage_types):
        """Index over trimmed storage types"""
        storage_types_list = [self.show(request, storage_type, True)
                             for storage_type in storage_types]
        return dict(storage_types=storage_types_list)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime
from sqlalchemy import Integer, MetaData, String
from sqlalchemy import Table, Index

def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine;
    # bind migrate_engine to your metadata
    meta = MetaData()
    meta.bind = migrate_engine

    storage_group = Table(
        'storage_groups', meta,
        Column('id', Integer, primary_key=True, nullable=False),
        Column('name', String(length=255), nullable=False),
        Column('storage_class', String(length=255), nullable=False),
        Column('friendly_name', String(length=255), nullable=False),
        Column('rule_id', Integer, index=True, nullable=False),
        Column('drive_extended_threshold', Integer, default=0, nullable=False),
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime(timezone=False)),
        Column('deleted', Boolean(create_constraint=True, name=None)),
    )

    try:
        storage_group.create()
    except Exception:
        meta.drop_all(tables=[storage_group])
        raise

def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    storage_group = Table('storage_groups',
                    meta,
                    autoload=True)
    storage_group.drop()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
from webob import exc
import re

from vsm.api import common
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import exception
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import servers as servers_views
from vsm.openstack.common import jsonutils
from vsm import utils
from vsm import conductor
from vsm import scheduler
from vsm.scheduler import rpcapi as scheduler_rpcapi

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_server(elem, detailed=False):
    elem.set('id')
    elem.set('host')
    elem.set('primary_public_ip')
    elem.set('secondary_public_ip')
    elem.set('cluster_ip')
    elem.set('raw_ip')
    #elem.set('zone')
    elem.set('zone_id')
    elem.set('osds')
    elem.set('type')
    elem.set('status')

    if detailed:
        pass

    #xmlutil.make_links(elem, 'links')

def _translate_server_summary_view(context, server):
    if not server:
        return {"id": "",
                "host": "",
                "primary_public_ip": "",
                "secondary_public_ip": "",
                "cluster_ip": "",
                "raw_ip":"",
                "ceph_ver":"",
                "zone_id": "",
                "osds": "",
                "type": "",
                "status": ""}
    d = {"id": server["id"],
        "host": server["host"],
        "primary_public_ip": server["primary_public_ip"],
        "secondary_public_ip": server["secondary_public_ip"],
        "cluster_ip": server["cluster_ip"],
        "raw_ip":"192.168.1.3,192.168.2.3,192.168.3.3",
        "zone_id": server["zone_id"],
        "ceph_ver":server["ceph_ver"],
        "osds": server['data_drives_number'],
        "type": server['type'],
        "status": server['status']}
    if d['type'] is None:
        d['type'] = ""
    return d

server_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class ServerTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('server', selector='server')
        make_server(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=server_nsmap)

class ServersTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('servers')
        elem = xmlutil.SubTemplateElement(root, 'server', selector='servers')
        make_server(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=server_nsmap)

class ServersController(wsgi.Controller):
    """The Servers API controller for the OpenStack API."""
    _view_builder_class = servers_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.scheduler_rpcapi = scheduler_rpcapi.SchedulerAPI()
        self.ext_mgr = ext_mgr
        super(ServersController, self).__init__()

    def _get_server_search_options(self):
        """Return server search options allowed by non-admin."""
        return ('id', 'name', 'public_ip')

    @wsgi.serializers(xml=ServersTemplate)
    def index(self, req):
        """Get server list."""
        #search_opts = {}
        #search_opts.update(req.GET)
        context = req.environ['vsm.context']
        #remove_invalid_options(context,
        #                        search_opts,
        #                        self._get_server_search_options)
        servers = self.conductor_api.get_server_list(context)
        #server_list = servers.values()
        #sorted_servers = sorted(server_list,
        #                      key=lambda item: item['id'])

        return self._view_builder.index(servers)

    @wsgi.serializers(xml=ServersTemplate)
    def update(self, req, id, body):
        """update server."""
        LOG.info("CEPH_LOG server update body: %s" % body)
        return {"server": {"id":1,"name":"2"}}

    @wsgi.serializers(xml=ServersTemplate)
    def show(self, req, id):
        """update cluster."""
        context = req.environ['vsm.context']
        server = self.conductor_api.get_server(context, id)
        LOG.info("CEPH_LOG cluster show id: %s" % id)
        return {"server": _translate_server_summary_view(context, server)}

    def add(self, req, body=None):
        LOG.info('CEPH_LOG add-server body %s ' % body)
        context = req.environ['vsm.context']

        self.scheduler_api.add_servers(context, body)
        return webob.Response(status_int=202)

    def remove(self, req, body=None):
        LOG.info('CEPH_LOG remove body %s ' % body)
        context = req.environ['vsm.context']

        self.scheduler_api.remove_servers(context, body)
        return webob.Response(status_int=202)

    def reset_status(self, req, body=None):
        LOG.debug('reset_status = %s' % body)
        context = req.environ['vsm.context']
        init_node_id = body.get('servers', None)
        if not init_node_id:
            return webob.Response(status_int=202)

        node_ref = self.conductor_api.init_node_get_by_id(context,
            init_node_id)
        if node_ref:
            if node_ref['status'] == 'available':
                return webob.Response(status_int=202)
            if node_ref['status'].lower().find('error') != -1:
                return webob.Response(status_int=202)

        else:
            return webob.Response(status_int=202)

        self.conductor_api.init_node_update_status_by_id(context,
            init_node_id, 'Active')

        return {'status': 'ok'}
        return webob.Response(status_int=202)

    def start(self, req, body=None):
        LOG.info('DEBUG start-server body %s ' % body)
        context = req.environ['vsm.context']

        self.scheduler_rpcapi.start_server(context, body)
        return webob.Response(status_int=202)

    def stop(self, req, body=None):
        LOG.info('DEBUG stop-server body %s ' % body)
        context = req.environ['vsm.context']

        self.scheduler_rpcapi.stop_server(context, body)
        return webob.Response(status_int=202)

    def ceph_upgrade(self, req, body=None):
        LOG.info('DEBUG ceph_upgrade body %s ' % body)
        context = req.environ['vsm.context']
        ret = self.scheduler_rpcapi.ceph_upgrade(context, body)
        LOG.info('DEBUG ceph_upgrade ret %s ' % ret)
        return ret
        #return webob.Response(status_int=202)

def create_resource(ext_mgr):
    return wsgi.Resource(ServersController(ext_mgr))

def remove_invalid_options(context, search_options, allowed_search_options):
    """Remove search options that are not valid for non-admin API/context."""
    if context.is_admin:
        # Allow all options
        return
    # Otherwise, strip out all unknown options
    unknown_options = [opt for opt in search_options
                       if opt not in allowed_search_options]
    bad_options = ", ".join(unknown_options)
    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
    LOG.debug(log_msg)
    for opt in unknown_options:
        del search_options[opt]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime, ForeignKey
from sqlalchemy import Integer, MetaData, String, Table
from vsm.db.sqlalchemy import models

def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine;
    # bind migrate_engine to your metadata
    meta = MetaData()
    meta.bind = migrate_engine

    init_nodes = Table(
        'init_nodes', meta,
        Column('id', Integer, primary_key=True, nullable=False),
        Column('host', String(length=255), nullable=True),
        Column('service_id', Integer(), nullable=True),
        Column('raw_ip', String(length=255), nullable=True),
        Column('primary_public_ip', String(length=255), nullable=True),
        Column('secondary_public_ip', String(length=255), nullable=True),
        Column('cluster_ip', String(length=255), nullable=True),
        Column('zone_id', Integer(), nullable=True),
        Column('mds', String(length=255), nullable=True),
        Column('type', String(length=255), nullable=True),
        Column('cluster_id', Integer, ForeignKey(models.Cluster.id), nullable=True),
        Column('data_drives_number', String(length=255), nullable=True),
        Column('status', String(length=255), nullable=True),
        Column('pre_status', String(length=255), nullable=True),
        Column('id_rsa_pub', String(length=1024), nullable=True),
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime(timezone=False)),
        Column('deleted', Boolean(create_constraint=True, name=None)),
    )

    try:
        init_nodes.create()
    except Exception:
        meta.drop_all(tables=[init_nodes])
        raise

def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    init_nodes = Table('init_nodes',
                        meta,
                        autoload=True)
    init_nodes.drop()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "zones"

    def basic(self, zone):
        #LOG.info("CEPH_LOG zone api view %s " % zone)
        return {
            "zone": {
                "id":zone['id'], 
                "name": zone["name"], 
            }
        }

    def index(self, zones):
        """Show a list of servers without many details."""
        return self._list_view(self.basic, zones)

    def _list_view(self, func, zones):
        """Provide a view for a list of servers."""
        zone_list = [func(zone)["zone"] for zone in zones]
        zones_dict = dict(zones=zone_list)
        return zones_dict




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.


from vsm.api.openstack import wsgi
from vsm import flags


from vsm import scheduler
from vsm import db
from vsm.api.views import ec_profiles as ec_profile_views
from vsm.api import xmlutil
from vsm.openstack.common import log as logging
LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_ec_profile(elem, detailed=False):
    elem.set('id')
    elem.set('name')
    elem.set('plugin')
    elem.set('plugin')
    elem.set('plugin_path')
    elem.set('plugin_kv_pair')
    elem.set('pg_num')
    if detailed:
        pass
ec_profile_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}



class ECProfilesTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('ec_profiles')
        elem = xmlutil.SubTemplateElement(root, 'ec_profiles', selector='ec_profile')
        make_ec_profile(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=ec_profile_nsmap)


class ECProfileController(wsgi.Controller):
    """The ECProfile API controller for the ECProfile API."""
    _view_builder_class = ec_profile_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        super(ECProfileController, self).__init__()

    def detail(self, req):
        """Returns the list of ecprofiles."""
        LOG.info("Get a list of ecprofiles")
        context = req.environ['vsm.context']
        ec_profiles = db.ec_profile_get_all(context)
        return self._view_builder.detail(req, ec_profiles)

    def ec_profile_create(self, req, body=None):
        LOG.info('CEPH_LOG ec_profile_create body %s ' % body)
        context = req.environ['vsm.context']
        for profile in body.get('ec_profiles'):
            db.ec_profile_update_or_create(context,profile)
        return {'message':{'error_code':'','error_msg':'','info':'Add rbd group success!'}}

    def ec_profile_update(self, req, body=None):
        LOG.info('CEPH_LOG ec_profile_update body %s ' % body)
        context = req.environ['vsm.context']
        for profile in body.get('ec_profiles'):
            db.ec_profile_update_or_create(context,profile)
        return {'message':{'error_code':'','error_msg':'','info':'Add rbd group success!'}}


    def ec_profiles_remove(self, req, body=None):
        LOG.info('CEPH_LOG rbd_group_remove body %s ' % body)
        context = req.environ['vsm.context']
        message = {'message':{'error_code':'','error_msg':'','info':''}}
        success_ids = []
        for profile_id in body.get('ec_profiles'):
            db.ec_profile_remove(context,profile_id)
            success_ids.append(profile_id)
        message['message']['info'] += 'EC profiles %s removed success!'%(','.join(success_ids))
        return message


def create_resource(ext_mgr):
    return wsgi.Resource(ECProfileController(ext_mgr))




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""The storage type & storage types extra specs extension."""

from webob import exc

from vsm.api.openstack import wsgi
from vsm.api.views import types as views_types
from vsm.api import xmlutil
from vsm import exception

def make_voltype(elem):
    elem.set('id')
    elem.set('name')
    extra_specs = xmlutil.make_flat_dict('extra_specs', selector='extra_specs')
    elem.append(extra_specs)

class HardwareTypeTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_type', selector='storage_type')
        make_voltype(root)
        return xmlutil.MasterTemplate(root, 1)

class HardwareTypesTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('storage_types')
        elem = xmlutil.SubTemplateElement(root, 'storage_type',
                                          selector='storage_types')
        make_voltype(elem)
        return xmlutil.MasterTemplate(root, 1)

class HardwareTypesController(wsgi.Controller):
    """The storage types API controller for the OpenStack API."""

    _view_builder_class = views_types.ViewBuilder

    @wsgi.serializers(xml=HardwareTypesTemplate)
    def index(self, req):
        """Returns the list of storage types."""
        context = req.environ['vsm.context']
        vol_types = storage_types.get_all_types(context).values()
        return self._view_builder.index(req, vol_types)

    @wsgi.serializers(xml=HardwareTypeTemplate)
    def show(self, req, id):
        """Return a single storage type item."""
        context = req.environ['vsm.context']

        try:
            vol_type = storage_types.get_storage_type(context, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        # TODO(bcwaldon): remove str cast once we use uuids
        vol_type['id'] = str(vol_type['id'])
        return self._view_builder.show(req, vol_type)

def create_resource():
    return wsgi.Resource(HardwareTypesController())



# vim: tabstop=4 shiftwidth=4 softtabstop=4

#    Copyright 2011 Cloudscaling Group, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
The MatchMaker classes should except a Topic or Fanout exchange key and
return keys for direct exchanges, per (approximate) AMQP parlance.
"""

import contextlib
import itertools
import json

import eventlet
from oslo.config import cfg

from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging

matchmaker_opts = [
    # Matchmaker ring file
    cfg.StrOpt('matchmaker_ringfile',
               default='/etc/nova/matchmaker_ring.json',
               help='Matchmaker ring file (JSON)'),
    cfg.IntOpt('matchmaker_heartbeat_freq',
               default='300',
               help='Heartbeat frequency'),
    cfg.IntOpt('matchmaker_heartbeat_ttl',
               default='600',
               help='Heartbeat time-to-live.'),
]

CONF = cfg.CONF
CONF.register_opts(matchmaker_opts)
LOG = logging.getLogger(__name__)
contextmanager = contextlib.contextmanager

class MatchMakerException(Exception):
    """Signified a match could not be found."""
    message = _("Match not found by MatchMaker.")

class Exchange(object):
    """
    Implements lookups.
    Subclass this to support hashtables, dns, etc.
    """
    def __init__(self):
        pass

    def run(self, key):
        raise NotImplementedError()

class Binding(object):
    """
    A binding on which to perform a lookup.
    """
    def __init__(self):
        pass

    def test(self, key):
        raise NotImplementedError()

class MatchMakerBase(object):
    """
    Match Maker Base Class.
    Build off HeartbeatMatchMakerBase if building a
    heartbeat-capable MatchMaker.
    """
    def __init__(self):
        # Array of tuples. Index [2] toggles negation, [3] is last-if-true
        self.bindings = []

        self.no_heartbeat_msg = _('Matchmaker does not implement '
                                  'registration or heartbeat.')

    def register(self, key, host):
        """
        Register a host on a backend.
        Heartbeats, if applicable, may keepalive registration.
        """
        pass

    def ack_alive(self, key, host):
        """
        Acknowledge that a key.host is alive.
        Used internally for updating heartbeats,
        but may also be used publically to acknowledge
        a system is alive (i.e. rpc message successfully
        sent to host)
        """
        pass

    def is_alive(self, topic, host):
        """
        Checks if a host is alive.
        """
        pass

    def expire(self, topic, host):
        """
        Explicitly expire a host's registration.
        """
        pass

    def send_heartbeats(self):
        """
        Send all heartbeats.
        Use start_heartbeat to spawn a heartbeat greenthread,
        which loops this method.
        """
        pass

    def unregister(self, key, host):
        """
        Unregister a topic.
        """
        pass

    def start_heartbeat(self):
        """
        Spawn heartbeat greenthread.
        """
        pass

    def stop_heartbeat(self):
        """
        Destroys the heartbeat greenthread.
        """
        pass

    def add_binding(self, binding, rule, last=True):
        self.bindings.append((binding, rule, False, last))

    #NOTE(ewindisch): kept the following method in case we implement the
    #                 underlying support.
    #def add_negate_binding(self, binding, rule, last=True):
    #    self.bindings.append((binding, rule, True, last))

    def queues(self, key):
        workers = []

        # bit is for negate bindings - if we choose to implement it.
        # last stops processing rules if this matches.
        for (binding, exchange, bit, last) in self.bindings:
            if binding.test(key):
                workers.extend(exchange.run(key))

                # Support last.
                if last:
                    return workers
        return workers

class HeartbeatMatchMakerBase(MatchMakerBase):
    """
    Base for a heart-beat capable MatchMaker.
    Provides common methods for registering,
    unregistering, and maintaining heartbeats.
    """
    def __init__(self):
        self.hosts = set()
        self._heart = None
        self.host_topic = {}

        super(HeartbeatMatchMakerBase, self).__init__()

    def send_heartbeats(self):
        """
        Send all heartbeats.
        Use start_heartbeat to spawn a heartbeat greenthread,
        which loops this method.
        """
        for key, host in self.host_topic:
            self.ack_alive(key, host)

    def ack_alive(self, key, host):
        """
        Acknowledge that a host.topic is alive.
        Used internally for updating heartbeats,
        but may also be used publically to acknowledge
        a system is alive (i.e. rpc message successfully
        sent to host)
        """
        raise NotImplementedError("Must implement ack_alive")

    def backend_register(self, key, host):
        """
        Implements registration logic.
        Called by register(self,key,host)
        """
        raise NotImplementedError("Must implement backend_register")

    def backend_unregister(self, key, key_host):
        """
        Implements de-registration logic.
        Called by unregister(self,key,host)
        """
        raise NotImplementedError("Must implement backend_unregister")

    def register(self, key, host):
        """
        Register a host on a backend.
        Heartbeats, if applicable, may keepalive registration.
        """
        self.hosts.add(host)
        self.host_topic[(key, host)] = host
        key_host = '.'.join((key, host))

        self.backend_register(key, key_host)

        self.ack_alive(key, host)

    def unregister(self, key, host):
        """
        Unregister a topic.
        """
        if (key, host) in self.host_topic:
            del self.host_topic[(key, host)]

        self.hosts.discard(host)
        self.backend_unregister(key, '.'.join((key, host)))

        LOG.info(_("Matchmaker unregistered: %s, %s" % (key, host)))

    def start_heartbeat(self):
        """
        Implementation of MatchMakerBase.start_heartbeat
        Launches greenthread looping send_heartbeats(),
        yielding for CONF.matchmaker_heartbeat_freq seconds
        between iterations.
        """
        if len(self.hosts) == 0:
            raise MatchMakerException(
                _("Register before starting heartbeat."))

        def do_heartbeat():
            while True:
                self.send_heartbeats()
                eventlet.sleep(CONF.matchmaker_heartbeat_freq)

        self._heart = eventlet.spawn(do_heartbeat)

    def stop_heartbeat(self):
        """
        Destroys the heartbeat greenthread.
        """
        if self._heart:
            self._heart.kill()

class DirectBinding(Binding):
    """
    Specifies a host in the key via a '.' character
    Although dots are used in the key, the behavior here is
    that it maps directly to a host, thus direct.
    """
    def test(self, key):
        if '.' in key:
            return True
        return False

class TopicBinding(Binding):
    """
    Where a 'bare' key without dots.
    AMQP generally considers topic exchanges to be those *with* dots,
    but we deviate here in terminology as the behavior here matches
    that of a topic exchange (whereas where there are dots, behavior
    matches that of a direct exchange.
    """
    def test(self, key):
        if '.' not in key:
            return True
        return False

class FanoutBinding(Binding):
    """Match on fanout keys, where key starts with 'fanout.' string."""
    def test(self, key):
        if key.startswith('fanout~'):
            return True
        return False

class StubExchange(Exchange):
    """Exchange that does nothing."""
    def run(self, key):
        return [(key, None)]

class RingExchange(Exchange):
    """
    Match Maker where hosts are loaded from a static file containing
    a hashmap (JSON formatted).

    __init__ takes optional ring dictionary argument, otherwise
    loads the ringfile from CONF.mathcmaker_ringfile.
    """
    def __init__(self, ring=None):
        super(RingExchange, self).__init__()

        if ring:
            self.ring = ring
        else:
            fh = open(CONF.matchmaker_ringfile, 'r')
            self.ring = json.load(fh)
            fh.close()

        self.ring0 = {}
        for k in self.ring.keys():
            self.ring0[k] = itertools.cycle(self.ring[k])

    def _ring_has(self, key):
        if key in self.ring0:
            return True
        return False

class RoundRobinRingExchange(RingExchange):
    """A Topic Exchange based on a hashmap."""
    def __init__(self, ring=None):
        super(RoundRobinRingExchange, self).__init__(ring)

    def run(self, key):
        if not self._ring_has(key):
            LOG.warn(
                _("No key defining hosts for topic '%s', "
                  "see ringfile") % (key, )
            )
            return []
        host = next(self.ring0[key])
        return [(key + '.' + host, host)]

class FanoutRingExchange(RingExchange):
    """Fanout Exchange based on a hashmap."""
    def __init__(self, ring=None):
        super(FanoutRingExchange, self).__init__(ring)

    def run(self, key):
        # Assume starts with "fanout~", strip it for lookup.
        nkey = key.split('fanout~')[1:][0]
        if not self._ring_has(nkey):
            LOG.warn(
                _("No key defining hosts for topic '%s', "
                  "see ringfile") % (nkey, )
            )
            return []
        return map(lambda x: (key + '.' + x, x), self.ring[nkey])

class LocalhostExchange(Exchange):
    """Exchange where all direct topics are local."""
    def __init__(self, host='localhost'):
        self.host = host
        super(Exchange, self).__init__()

    def run(self, key):
        return [('.'.join((key.split('.')[0], self.host)), self.host)]

class DirectExchange(Exchange):
    """
    Exchange where all topic keys are split, sending to second half.
    i.e. "compute.host" sends a message to "compute.host" running on "host"
    """
    def __init__(self):
        super(Exchange, self).__init__()

    def run(self, key):
        e = key.split('.', 1)[1]
        return [(key, e)]

class MatchMakerRing(MatchMakerBase):
    """
    Match Maker where hosts are loaded from a static hashmap.
    """
    def __init__(self, ring=None):
        super(MatchMakerRing, self).__init__()
        self.add_binding(FanoutBinding(), FanoutRingExchange(ring))
        self.add_binding(DirectBinding(), DirectExchange())
        self.add_binding(TopicBinding(), RoundRobinRingExchange(ring))

class MatchMakerLocalhost(MatchMakerBase):
    """
    Match Maker where all bare topics resolve to localhost.
    Useful for testing.
    """
    def __init__(self, host='localhost'):
        super(MatchMakerLocalhost, self).__init__()
        self.add_binding(FanoutBinding(), LocalhostExchange(host))
        self.add_binding(DirectBinding(), DirectExchange())
        self.add_binding(TopicBinding(), LocalhostExchange(host))

class MatchMakerStub(MatchMakerBase):
    """
    Match Maker where topics are untouched.
    Useful for testing, or for AMQP/brokered queues.
    Will not work where knowledge of hosts is known (i.e. zeromq)
    """
    def __init__(self):
        super(MatchMakerLocalhost, self).__init__()

        self.add_binding(FanoutBinding(), StubExchange())
        self.add_binding(DirectBinding(), StubExchange())
        self.add_binding(TopicBinding(), StubExchange())



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import license as license_views
from vsm import conductor

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

def make_license(elem, detailed=False):
    elem.set('license_accept')

    if detailed:
        pass

license_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class LicenseTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('license', selector='license')
        make_license(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=license_nsmap)

class LicensesTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('licenses')
        elem = xmlutil.SubTemplateElement(root, 'licenses', selector='licenses')
        make_license(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=license_nsmap)

class LicenseController(wsgi.Controller):
    """The License controller for the OpenStack API."""
    _view_builder_class = license_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.ext_mgr = ext_mgr
        super(LicenseController, self).__init__()

    def license_status_get(self, req):
        context = req.environ['vsm.context']
        ret = self.conductor_api.license_status_get(context)
        #if ret is None:
        #    LOG.error("In vsm api, licenses.py, ret is None")
        #else:
        #    LOG.error("Gao Gao, ret = %s" % ret)
        #    LOG.error("In vsm api, licenses.py, ret is not None")
        return ret

    def license_status_create(self, req, body=None):
        context = req.environ['vsm.context']
    
        #FIXME(fenqian): Hard code for the id, because we only have
        #one item in database.
        kargs = {'id': 1,
                 'license_accept': body['value']}
        ret = self.conductor_api.license_status_create(context, values=kargs)
        return ret

    def license_status_update(self, req, body=None):
        context = req.environ['vsm.context']
        value = body['value']
        ret = self.conductor_api.license_status_update(context, value)
        return ret

def create_resource(ext_mgr):
    return wsgi.Resource(LicenseController(ext_mgr))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Drivers for Agent Services.
"""

from oslo.config import cfg
from vsm import flags
from vsm import utils
from vsm.physical import driver
from vsm.physical.worker import cpu as cpu_worker
from vsm.physical.worker import network as network_worker
from vsm.physical.worker import disk as disk_worker
from vsm.physical.worker import memory as memory_worker
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class GeneralDriver(driver.PhysicalDriver):
    """General class for Physical Driver.

    """

    def __init__(self):
        super(GeneralDriver, self).__init__()
        self._cpu_worker = cpu_worker.CPUWorker()
        self._network_worker = network_worker.NetworkWorker()
        self._disk_worker = disk_worker.DiskWorker()
        self._memory_worker = memory_worker.MemoryWorker()

    #def init_host(self):
    #    """Initialize anything that is necessary for the driver."""
    #    #cpu_cores = self._cpu_worker.get_cpu_cores()
    #    #LOG.info("general:cpu cores:%s" % cpu_cores)
    #    ips = self._network_worker.get_ips()
    #    LOG.info("general:ips:%s" % ips)
    #    disk_partitions = self._disk_worker.get_disk_partitions()
    #    LOG.info("general:disk_partitions:%s" % disk_partitions)
    #    mem_total = self._memory_worker.get_mem_total()
    #    LOG.info("general:mem_total:%s" % mem_total)
    #    hostname = self._host_worker.get_hostname()
    #    LOG.info("general:hostname:%s" % hostname)

    #def get_disk_usage(self, path):
    #    return self._disk_worker.get_disk_usage(path)




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import and_, String, Column, MetaData, select, Table, Integer

def upgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    storage_pools = Table('storage_pools', meta, autoload=True)
    ec_status = Column('ec_status', String(length=255)) 

    storage_pools.create_column(ec_status)

    storage_groups = Table('storage_groups', meta, autoload=True)


def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    storage_pools = Table('storage_pools', meta, autoload=True)

    storage_pools.drop_column('ec_status')




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Drivers for get diff physical devices.
"""

from oslo.config import cfg
from vsm import flags
from vsm import utils
from vsm.openstack.common import importutils
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

physical_driver_opts = [
    cfg.StrOpt('physical_driver',
               default='vsm.physical.general.GeneralDriver',
               help='Driver to use for controller storage system.'
                    'include: vsm.physical.general.GeneralDriver'),
]

CONF = cfg.CONF
CONF.register_opts(physical_driver_opts)

def load_physical_driver(physical_driver=None):
    """Load ad physical driver module.

    Load the physical driver module specified by the physical_driver
    configuration option or, if supplied, the driver name supplied
    as an argument.

    :param physical_driver: a physical driver name to override the config opt.
    :returns: a PhysicalDriver interface.
    """
    if not physical_driver:
        physical_driver = CONF.physical_driver

    if not physical_driver:
        LOG.warn(_("Physical driver option required, but not specified"))
        raise

    try:
        # If just write driver.CephDriver
        driver = importutils.import_object_ns('vsm.physical',
                                              physical_driver)
        return utils.check_isinstance(driver, PhysicalDriver)
    except ImportError:
        try:
            # If vsm.physical.driver.CephDriver
            driver = importutils.import_object(physical_driver)
            return utils.check_isinstance(driver, PhysicalDriver)
        except ImportError:
            LOG.exception(_("Unable to load the physical driver"))
            raise

class PhysicalDriver(object):
    """Basic class for Physical Driver.

    The physical Driver mainly use to manage physical nodes in
    the cluster.

    Now we implement the generaldriver which just can do what general
    physical servers do. For example, there is not BMC hardware on
    the node.

    """

    def __init__(self):
        pass

    #def init_host(self, host):
    #    """Initialize anything that is necessary for the driver."""
    #    raise NotImplementedError()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob
import commands
from webob import exc
from vsm.api import extensions
from vsm.api.openstack import wsgi
from vsm.agent import storagepoolusage
from vsm.agent import appnodes
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import poolusages as poolusages_views
from vsm.openstack.common.gettextutils import _
from vsm.scheduler.api import API as scheduler_api
from vsm import db
from vsm import utils

"""Storage pool usage API"""

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_poolusage(elem, detailed=False):
    elem.set('id')
    elem.set('vsmapp_id')
    elem.set('pool_id')
    elem.set('attach_status')
    elem.set('attach_at')

    if detailed:
        pass

poolusage_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class PoolUsageTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('poolusage', selector='poolusage')
        make_poolusage(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=poolusage_nsmap)

class PoolUsagesTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('poolusages')
        elem = xmlutil.SubTemplateElement(root, 'poolusage', selector='poolusages')
        make_poolusage(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=poolusage_nsmap)

class PoolUsagesController(wsgi.Controller):
    """The storage pool usage API controller."""
    _view_builder_class = poolusages_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.ext_mgr = ext_mgr
        self.scheduler_api = scheduler_api()
        super(PoolUsagesController, self).__init__()

    @wsgi.serializers(xml=PoolUsagesTemplate)
    def index(self, req):
        """Returns a summary list of pool usages."""
        return self._get_usages(req)

    def _get_usages(self, req):
        """Returns a list of pool usages, transformed through view builder."""
        context = req.environ['vsm.context']
        usages = storagepoolusage.get_all(context)
        usage_view = self._view_builder.index(usages)
        return usage_view

    @wsgi.response(202)
    def create(self, req, body):
        LOG.debug(_('Creating new pool usages %s'), body)
        LOG.info(' Creating new pool usages')

        context = req.environ['vsm.context']
        pools = body['poolusages']

        host_list = []
        count_crudini = 0
        count_ssh = 0
        host = []
        for pool in pools:
            appnode_id = pool['appnode_id']
            appnode = db.appnodes_get_by_id(context, appnode_id)
            auth_url = appnode['os_auth_url']
            os_controller_host = auth_url.split(":")[1][2:]
            ssh_user = appnode['ssh_user']
            if os_controller_host not in host:
                host.append(os_controller_host)
                result, err = utils.execute('check_xtrust_crudini',
                                            ssh_user, os_controller_host,
                                            run_as_root = True)
                LOG.info("==============result: %s" % result)
                LOG.info("==============err: %s" % err)
                if "command not found" in err:
                    count_crudini = count_crudini + 1
                    host_list.append(host)
                if "Permission denied" in err or "No passwd entry" in err:
                    count_ssh = count_ssh + 1
                    host_list.append(host)

        if count_crudini != 0:
            return {'status': 'bad', 'host': list(set(host_list))}
        elif count_ssh != 0:
            return {'status': 'unreachable', 'host': list(set(host_list))}
        else:
            info = storagepoolusage.create(context, pools)
            LOG.info(' pools_info = %s' % info)
            self.scheduler_api.present_storage_pools(context, info)
            return {'status': 'ok', 'host': host_list}

    @wsgi.serializers(xml=PoolUsagesTemplate)
    def update(self, req, id, body):
        LOG.debug(_('Updating pool usage by vsmapp id %s' % id))
        context = req.environ['vsm.context']

        if not self.is_valid_body(body, 'poolusages'):
            raise exc.HTTPBadRequest()

        body = body.get('poolusages')
        LOG.debug('PUT body: %s' % body)

        if id is None:
            expl = _('Request body and URI mismatch: No vsmapp_id')
            raise webob.exc.HTTPBadRequest(explanation=expl)
        # convert from unicode to str
        id = str(id)
        status = body.get('attach_status', None)
        terminate = body.get('is_terminate', None)

        if not status and not terminate:
            expl = _('Request body and URI mismatch: attach_status or is_terminate required.')
            raise webob.exc.HTTPBadRequest(explanation=expl)

        storagepoolusage.update(context, id, status, terminate)
        return webob.Response(status_int=201)

    @wsgi.serializers(xml=PoolUsagesTemplate)
    def delete(self, req, id):
        """delete pool usage by id."""
        LOG.debug(_('Removing pool ussage by id %s' % id))
        context = req.environ['vsm.context']
        if id is None:
            expl = _('Request body and URI mismatch')
            raise webob.exc.HTTPBadRequest(explanation=expl)
        id = str(id)
        storagepoolusage.destroy(context, id)
        return webob.Response(status_int=201)

    def revoke_pool(self, req, body):
        """
        revoke pool from openstack cinder.conf.
        config the cinder.conf.
        delete cinder type.
        delete ceph auth caps.
        delete pool usage as deleted.
        :param body:
        {
            "poolusage": {
                "id": "1"
            }
        }
        """

        LOG.info("Revoke pool from openstack cinder")
        context = req.environ['vsm.context']
        body = body.get('poolusage')

        id = body.get("id", "")
        try:
            self.scheduler_api.revoke_storage_pool(context, id)
        except:
            raise webob.Response(status=400)

# class Poolusages(extensions.ExtensionDescriptor):
#     """storage pool usage extension."""
#
#     name = 'Poolusages'
#     alias = 'poolusages'
#     namespace = 'http://docs.openstack.org/storage/ext/poolusages/api/v1'
#     updated = '2014-03-19T00:00:00+00:00'
#
#     def get_resources(self):
#         resources = []
#         res = extensions.ResourceExtension(Poolusages.alias, PoolUsagesController(""))
#         resources.append(res)
#         return resources

def create_resource(ext_mgr):
    return wsgi.Resource(PoolUsagesController(ext_mgr))


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import osds as osds_views
from vsm.api.views import summary as summary_view
from vsm import conductor
from vsm import scheduler
from vsm import exception
from vsm import db

LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_osd(elem, detailed=False):
    elem.set('id')
    elem.set('osd_name')
    elem.set('state')
    elem.set('operation_status')
    elem.set('weight')
    elem.set('device_id')
    elem.set('service_id')

    if detailed:
        pass

osd_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class OSDTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('osd', selector='osd')
        make_osd(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=osd_nsmap)

class OSDsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('osds')
        elem = xmlutil.SubTemplateElement(root, 'osd', selector='osds')
        make_osd(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=osd_nsmap)

class Controller(wsgi.Controller):
    """The OSDs API controller for the OpenStack API."""
    _view_builder_class = osds_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        super(Controller, self).__init__()

    #def _get_zone_search_options(self):
    #    """Return zone search options allowed by non-admin."""
    #    return ('id', 'name', 'public_ip')

    def _get_osd(self, context, req, id):
        """Utility function for looking up an instance by id."""
        try:
            osd = self.conductor_api.osd_get(context, id)
        except exception.NotFound:
            msg = _("OSD could not be found")
            raise exc.HTTPNotFound(explanation=msg)
        return osd

    @wsgi.serializers(xml=OSDsTemplate)
    def show(self, req, id):
        """Return data about the given osd."""
        context = req.environ['vsm.context']

        try:
            osd = self._get_osd(context, req, id)
            error = self.conductor_api.ceph_error(context)
            LOG.info('JIYOU show')
            if error:
                osd['state'] = error
                osd['operation_status'] = error
                osd['device']['state'] = error
                osd['device']['journal_state'] = error
                osd['device']['osd_state'] = error
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return {'osd': osd}

    @wsgi.serializers(xml=OSDsTemplate)
    def index(self, req):
        """Get osd list."""
        context = req.environ['vsm.context']
        error = self.conductor_api.ceph_error(context)
        osds = self.conductor_api.osd_state_get_all(context)
        if error:
            for osd in osds:
                osd['state'] = error
                osd['operation_status'] = error
                osd['device']['state'] = error
                osd['device']['journal_state'] = error
                osd['device']['osd_state'] = error

        LOG.info('vsm/api/v1/osds.py osds:%s' % osds)

        return self._view_builder.index(req, osds)

    @wsgi.serializers(xml=OSDsTemplate)
    def detail(self, req):

        """Get osd list."""
        context = req.environ['vsm.context']
        service_id = req.GET.get('service_id', None)
        error = self.conductor_api.ceph_error(context)
        LOG.info('vsm/api/v1/osds.py detailed service_id:%s' % service_id)
        if service_id:
            osds = db.osd_get_by_service_id(context, service_id)
            if len(osds) > 0:
                LOG.info('device=%s'%osds[0].device)
        else:
            limit = req.GET.get('limit', None)
            marker = req.GET.get('marker', None)
            sort_keys = req.GET.get('sort_keys', None)
            sort_dir = req.GET.get('sort_dir', None)
            osds = self.conductor_api.osd_state_get_all(context, limit,
                                                        marker, sort_keys,
                                                        sort_dir)
        if error:
            for osd in osds:
                osd['state'] = error
                osd['operation_status'] = error
                osd['device']['state'] = error
                osd['device']['journal_state'] = error
                osd['device']['osd_state'] = error

        LOG.info('vsm/api/v1/osds.py detailed osds:%s' % osds)

        return self._view_builder.detail(req, osds)
    @wsgi.serializers(xml=OSDsTemplate)
    def detail_filter_and_sort(self, req):

        """Get osd list."""
        context = req.environ['vsm.context']
        limit = req.GET.get('limit', None)
        marker = req.GET.get('marker', None)
        sort_keys = req.GET.get('sort_keys', None)
        sort_dir = req.GET.get('sort_dir', None)
        search_opts = {
            'osd_name':req.GET.get('osd_name', ''),
            'server_name':req.GET.get('server_name', ''),
            'zone_name':req.GET.get('zone_name', ''),
            'state':req.GET.get('state', ''),
        }
        error = self.conductor_api.ceph_error(context)
        osds = self.conductor_api.osd_state_get_all(context, limit,
                                                    marker, sort_keys,
                                                    sort_dir,search_opts)

        if error:
            for osd in osds:
                osd['state'] = error
                osd['operation_status'] = error
                osd['device']['state'] = error
                osd['device']['journal_state'] = error
                osd['device']['osd_state'] = error

        LOG.info('vsm/api/v1/osds.py detail_filter_and_sort osds:%s' % osds)

        return self._view_builder.detail(req, osds)


    def refresh(self, req):
        """
        :param req:
        :return:
        refresh osd status
        """
        context = req.environ['vsm.context']
        self.scheduler_api.osd_refresh(context)

    #TODO scheduler exe
    @wsgi.response(204)
    def delete(self, req, id):
        """delete a osd."""
        LOG.info("osd_delete")
        context = req.environ['vsm.context']
        osd = self._get_osd(context, req, id)
        self.conductor_api.osd_delete(context, osd['id'])

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('remove')
    def _action_remove(self, req, id, body):
        context = req.environ['vsm.context']
        LOG.info("action_remove")
        self.scheduler_api.osd_remove(context, id)
        return webob.Response(status_int=202)

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('restart')
    def _action_restart(self, req, id, body):
        context = req.environ['vsm.context']
        LOG.info("action_restart")
        self.scheduler_api.osd_restart(context, id)
        return webob.Response(status_int=202)

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('restore')
    def _action_restore(self, req, id, body):
        context = req.environ['vsm.context']
        LOG.info("osd_restore osd_id = %s" % id)
        self.scheduler_api.osd_restore(context, id)
        return webob.Response(status_int=202)

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('add')
    def _action_add(self, req, id, body):
        context = req.environ['vsm.context']
        LOG.info("osd_add osd_id = %s" % id)
        self.scheduler_api.osd_add(context, id)
        return webob.Response(status_int=202)

    def add_new_disks_to_cluster(self, req, body):
        context = req.environ['vsm.context']
        LOG.info("osd_add body= %s" % body)
        self.scheduler_api.add_new_disks_to_cluster(context, body)
        return webob.Response(status_int=202)

    def add_batch_new_disks_to_cluster(self, req, body):
        context = req.environ['vsm.context']
        LOG.info("batch_osd_add body= %s" % body)
        ret = self.scheduler_api.add_batch_new_disks_to_cluster(context, body)
        LOG.info("batch_osd_add ret= %s" % ret)
        return ret

    def summary(self, req, cluster_id = None):
        #LOG.info('osd-summary body %s ' % body)
        context = req.environ['vsm.context']
        if cluster_id:
            sum = db.summary_get_by_cluster_id_and_type(context, cluster_id, 'osd')
        else:
            sum = db.summary_get_by_type_first(context, 'osd')
        #sum = db.summary_get_by_cluster_id_and_type(context, 1, 'osd')

        vb = summary_view.ViewBuilder()
        return vb.basic(sum, 'osd')

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))

#def remove_invalid_options(context, search_options, allowed_search_options):
#    """Remove search options that are not valid for non-admin API/context."""
#    if context.is_admin:
#        # Allow all options
#        return
#    # Otherwise, strip out all unknown options
#    unknown_options = [opt for opt in search_options
#                       if opt not in allowed_search_options]
#    bad_options = ", ".join(unknown_options)
#    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
#    LOG.debug(log_msg)
#    for opt in unknown_options:
#        del search_options[opt]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
WSGI middleware for OpenStack Hardware API.
"""

from vsm.api import extensions
import vsm.api.openstack
from vsm.api.v1 import limits
from vsm.api.v1 import types
from vsm.api import versions
from vsm.api.v1 import conductor
from vsm.api.v1 import storage_pool
from vsm.api.v1 import clusters
from vsm.api.v1 import servers
from vsm.api.v1 import zones
from vsm.api.v1 import agents
from vsm.api.v1 import osds
from vsm.api.v1 import mdses
from vsm.api.v1 import monitors 
from vsm.api.v1 import placement_groups
from vsm.api.v1 import storage_groups
from vsm.api.v1 import rbd_pools
from vsm.api.v1 import devices
from vsm.api.v1 import monitors
from vsm.api.v1 import vsm_settings
from vsm.api.v1 import vsms
from vsm.api.v1 import licenses
from vsm.api.v1 import performance_metrics
from vsm.api.contrib import poolusages
from vsm.api.v1 import ec_profiles
from vsm.api.v1 import rgw

from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)

class APIRouter(vsm.api.openstack.APIRouter):
    """
    Routes requests on the OpenStack API to the appropriate controller
    and method.
    """
    ExtensionManager = extensions.ExtensionManager

    def _setup_routes(self, mapper, ext_mgr):
        self.resources['versions'] = versions.create_resource()
        mapper.connect("versions", "/",
                       controller=self.resources['versions'],
                       action='show')

        mapper.redirect("", "/")

        self.resources['types'] = types.create_resource()
        mapper.resource("type", "types",
                        controller=self.resources['types'])

        self.resources['limits'] = limits.create_resource()
        mapper.resource("limit", "limits",
                        controller=self.resources['limits'])

        self.resources['conductor'] = conductor.create_resource(ext_mgr)
        mapper.resource("conductor", "conductor",
                        controller=self.resources['conductor'],
                        collection={'detail': 'get',
                                    'host_status': 'post',
                                    'resource_info':'post',},
                        member={'action': 'post'})

        self.resources['storage_pool'] = storage_pool.create_resource(ext_mgr)
        mapper.resource("storage_pool", "storage_pool",
                        controller=self.resources['storage_pool'],
                        collection={'detail': 'get',
                                    'test_scheduler': 'post',
                                    'resource_info':'post',
                                    'create': 'post',
                                    'get_storage_group_list': 'get',
                                    'get_pool_size_list': 'get',
                                    'list_storage_pool': 'get'},
                        member={'action': 'post'})

        # change name from storage_pool to storage_pools
        self.resources['storage_pools'] = storage_pool.create_resource(ext_mgr)
        mapper.resource("storage_pools", "storage_pools",
                        controller=self.resources['storage_pools'],
                        collection={'detail': 'get',
                                    'test_scheduler': 'post',
                                    'resource_info':'post',
                                    'create': 'post',
                                    'get_storage_group_list': 'get',
                                    'get_pool_size_list': 'get',
                                    'get_ec_profile_list': 'get',
                                    'add_cache_tier': 'post',
                                    'remove_cache_tier': 'post',
                                    'list_storage_pool': 'get'},
                        member={'action': 'post'})

        self.resources['clusters'] = clusters.create_resource(ext_mgr)
        mapper.resource("clusters", "clusters",
                        controller=self.resources['clusters'],
                        collection={'summary': 'get',
                                    'refresh': 'post',
                                    'import_ceph_conf': 'post',
                                    'integrate': 'post',
                                    'start_cluster': 'post',
                                    'stop_cluster': 'post',
                                    'get_ceph_health_list':'get',
                                    'check_pre_existing_cluster':'post',
                                    'import_cluster':'post',
                                    'detect_cephconf':'post',
                                    'detect_crushmap':'post',
                                    'get_crushmap_tree_data':'post',
                                    'get_service_list':'get'
                                    },
                        member={'action': 'post'})

        self.resources['servers'] = servers.create_resource(ext_mgr)
        mapper.resource("servers", "servers",
                        controller=self.resources['servers'],
                        collection={"add": "post",
                                    "remove": "post",
                                    "reset_status": "post",
                                    "start": "post",
                                    "stop": "post",
                                    "ceph_upgrade": "post"},
                        member={'action':'post'})

        self.resources['agents'] = agents.create_resource(ext_mgr)
        mapper.resource("agents", "agents",
                        controller=self.resources['agents'],
                        collection={'detail': 'get'},
                        member={'action':'post'})

        self.resources['zones'] = zones.create_resource(ext_mgr)
        mapper.resource("zones", "zones",
                        controller=self.resources['zones'],
                        collection={'osd_locations_choices': 'get',
                                    'get_zone_not_in_crush_list': 'get',
                                    'add_zone_to_crushmap_and_db': 'post',},
                        member={'action':'POST'})

        self.resources['osds'] = osds.create_resource(ext_mgr)
        mapper.resource("osds", "osds",
                        controller=self.resources['osds'],
                        collection={"summary": "get",
                                    "refresh": "post",
                                    "detail": "get",
                                    "add_batch_new_disks_to_cluster":"post",
                                    "add_new_disks_to_cluster":"post",
                                    "detail_filter_and_sort": "get"},
                        member={'action':'POST'})

        self.resources['mdses'] = mdses.create_resource(ext_mgr)
        mapper.resource("mdses", "mdses",
                        controller=self.resources['mdses'],
                        collection={"summary": "get",
                                    "detail": "get"},
                        member={'action':'POST'})

        self.resources['monitors'] = monitors.create_resource(ext_mgr)
        mapper.resource("monitors", "monitors",
                        controller=self.resources['monitors'],
                        collection={"summary": "get",
                                    "detail": "get"},
                        member={'action':'POST'})

        self.resources['vsms'] = vsms.create_resource(ext_mgr)
        mapper.resource("vsms", "vsms",
                        controller=self.resources['vsms'],
                        collection={"summary": "get"},
                        member={'action':'POST'})

        self.resources['storage_groups'] = storage_groups.create_resource(ext_mgr)
        mapper.resource("storage_groups", "storage_groups",
                        controller=self.resources['storage_groups'],
                        collection={"summary": "get",
                                    "create_with_takes":"post",
                                    "update_with_takes":"post",
                                    "detail": "get",
                                    "get_default_pg_num":"get"},
                        member={'action':'POST'})

        self.resources['placement_groups'] = placement_groups.create_resource(ext_mgr)
        mapper.resource("placement_groups", "placement_groups",
                        controller=self.resources['placement_groups'],
                        collection={"summary": "get",
                                    "detail": "get"},
                        member={'action':'POST'})

        self.resources['rbd_pools'] = rbd_pools.create_resource(ext_mgr)
        mapper.resource("rbd_pools", "rbd_pools",
                        controller=self.resources['rbd_pools'],
                        collection={"summary": "get",
                                    "detail": "get"},
                        member={'action':'POST'})

        self.resources['devices'] = devices.create_resource(ext_mgr)
        mapper.resource("devices", "devices",
                        controller=self.resources['devices'],
                        collection={"detail": "get",
                                    "get_available_disks":"get",
                                    "get_smart_info":"get",},
                        member={'action':'POST'})

        self.resources['licenses'] = licenses.create_resource(ext_mgr)
        mapper.resource("licenses", "licenses",
                        controller=self.resources['licenses'],
                        collection={'license_status_get': 'get',
                                    'license_status_create': 'post',
                                    'license_status_update': 'post'},
                        member={'action': 'post'})

        self.resources['vsm_settings'] = vsm_settings.create_resource(ext_mgr)
        mapper.resource("vsm_settings", "vsm_settings",
                        controller=self.resources['vsm_settings'],
                        collection={'detail': 'get',
                                    'create': 'post',
                                    'get_by_name': 'get'},
                        member={'action': 'post'})

        self.resources['performance_metrics'] = performance_metrics.create_resource(ext_mgr)
        mapper.resource("performance_metrics", "performance_metrics",
                        controller=self.resources['performance_metrics'],
                        collection={"get_list": "get",
                                    "get_metrics": "get",
                                    "get_metrics_all_types": "get",
                                    },
                        member={'action':'post'})

        self.resources['poolusages'] = poolusages.create_resource(ext_mgr)
        mapper.resource("poolusages", "poolusages",
                        controller=self.resources['poolusages'],
                        collection={'revoke_pool': "post"},
                        member={'action':'post'})

        self.resources['ec_profiles'] = ec_profiles.create_resource(ext_mgr)
        mapper.resource("ec_profiles", "ec_profiles",
                        controller=self.resources['ec_profiles'],
                        collection={
                            'detail':"get",
                            'ec_profile_create': "post",
                            'ec_profile_update': "post",
                            'ec_profiles_remove': "post",
                            },
                        member={'action': 'post'})

        self.resources['rgws'] = rgw.create_resource(ext_mgr)
        mapper.resource("rgws", "rgws",
                        controller=self.resources['rgws'],
                        collection={},
                        member={'action':'post'})


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from vsm.common import constant
from vsm import utils

def get_ceph_version():
    """

    Get the ceph version from ceph cluster.
    Run command "ceph --version" to get it.

    :return: string: version
    """

    args = ["ceph",
            "--version"]
    try:
        out, err = utils.execute(*args, run_as_root=True)
        version = out.split(" ")[2]
    except:
        version = ""
    return version

def get_ceph_version_code():
    """

    Get the ceph version first from funtion get_ceph_version.
    Then analyze and return the ceph version code.

    :return: string: code
    """

    code = "firefly"
    version = get_ceph_version()
    prefix_version = ".".join(version.split(".")[0:2])
    for k,v in constant.CEPH_VERSION_CODE:
        if prefix_version == k:
            code = v
    return code
